@@ -18,22 +18,27 @@
 //! Conversion between iceberg and avro schema.
 use std::collections::BTreeMap;
 
-use crate::spec::{
-    visit_schema, ListType, MapType, NestedField, NestedFieldRef, PrimitiveType, Schema,
-    SchemaVisitor, StructType, Type,
-};
-use crate::{ensure_data_valid, Error, ErrorKind, Result};
 use apache_avro::schema::{
-    DecimalSchema, FixedSchema, Name, RecordField as AvroRecordField, RecordFieldOrder,
-    RecordSchema, UnionSchema,
+    ArraySchema, DecimalSchema, FixedSchema, MapSchema, Name, RecordField as AvroRecordField,
+    RecordFieldOrder, RecordSchema, UnionSchema,
 };
 use apache_avro::Schema as AvroSchema;
 use itertools::{Either, Itertools};
 use serde_json::{Number, Value};
 
+use crate::spec::{
+    visit_schema, ListType, MapType, NestedField, NestedFieldRef, PrimitiveType, Schema,
+    SchemaVisitor, StructType, Type,
+};
+use crate::{ensure_data_valid, Error, ErrorKind, Result};
+
+const ELEMENT_ID: &str = "element-id";
 const FILED_ID_PROP: &str = "field-id";
+const KEY_ID: &str = "key-id";
+const VALUE_ID: &str = "value-id";
 const UUID_BYTES: usize = 16;
 const UUID_LOGICAL_TYPE: &str = "uuid";
+const MAP_LOGICAL_TYPE: &str = "map";
 // # TODO: https://github.com/apache/iceberg-rust/issues/86
 // This const may better to maintain in avro-rs.
 const LOGICAL_TYPE: &str = "logicalType";
@@ -123,8 +128,13 @@ impl SchemaVisitor for SchemaToAvroSchema {
             field_schema = avro_optional(field_schema)?;
         }
 
-        // TODO: We need to add element id prop here, but rust's avro schema doesn't support property except record schema.
-        Ok(Either::Left(AvroSchema::Array(Box::new(field_schema))))
+        Ok(Either::Left(AvroSchema::Array(ArraySchema {
+            items: Box::new(field_schema),
+            attributes: BTreeMap::from([(
+                ELEMENT_ID.to_string(),
+                Value::Number(Number::from(list.element_field.id)),
+            )]),
+        })))
     }
 
     fn map(
@@ -140,7 +150,19 @@ impl SchemaVisitor for SchemaToAvroSchema {
         }
 
         if matches!(key_field_schema, AvroSchema::String) {
-            Ok(Either::Left(AvroSchema::Map(Box::new(value_field_schema))))
+            Ok(Either::Left(AvroSchema::Map(MapSchema {
+                types: Box::new(value_field_schema),
+                attributes: BTreeMap::from([
+                    (
+                        KEY_ID.to_string(),
+                        Value::Number(Number::from(map.key_field.id)),
+                    ),
+                    (
+                        VALUE_ID.to_string(),
+                        Value::Number(Number::from(map.value_field.id)),
+                    ),
+                ]),
+            })))
         } else {
             // Avro map requires that key must be string type. Here we convert it to array if key is
             // not string type.
@@ -186,7 +208,13 @@ impl SchemaVisitor for SchemaToAvroSchema {
                 fields,
             )?;
 
-            Ok(Either::Left(AvroSchema::Array(item_avro_schema.into())))
+            Ok(Either::Left(AvroSchema::Array(ArraySchema {
+                items: Box::new(item_avro_schema),
+                attributes: BTreeMap::from([(
+                    LOGICAL_TYPE.to_string(),
+                    Value::String(MAP_LOGICAL_TYPE.to_string()),
+                )]),
+            })))
         }
     }
 
@@ -254,14 +282,28 @@ pub(crate) fn avro_fixed_schema(len: usize, logical_type: Option<&str>) -> Resul
         doc: None,
         size: len,
         attributes,
+        default: None,
     }))
 }
 
 pub(crate) fn avro_decimal_schema(precision: usize, scale: usize) -> Result<AvroSchema> {
+    // Avro decimal logical type annotates Avro bytes _or_ fixed types.
+    // https://avro.apache.org/docs/1.11.1/specification/_print/#decimal
+    // Iceberg spec: Stored as _fixed_ using the minimum number of bytes for the given precision.
+    // https://iceberg.apache.org/spec/#avro
     Ok(AvroSchema::Decimal(DecimalSchema {
         precision,
         scale,
-        inner: Box::new(AvroSchema::Bytes),
+        inner: Box::new(AvroSchema::Fixed(FixedSchema {
+            // Name is not restricted by the spec.
+            // Refer to iceberg-python https://github.com/apache/iceberg-python/blob/d8bc1ca9af7957ce4d4db99a52c701ac75db7688/pyiceberg/utils/schema_conversion.py#L574-L582
+            name: Name::new(&format!("decimal_{precision}_{scale}")).unwrap(),
+            aliases: None,
+            doc: None,
+            size: crate::spec::Type::decimal_required_bytes(precision as u32)? as usize,
+            attributes: Default::default(),
+            default: None,
+        })),
     }))
 }
 
@@ -287,8 +329,11 @@ pub(crate) trait AvroSchemaVisitor {
 
     fn union(&mut self, union: &UnionSchema, options: Vec<Self::T>) -> Result<Self::T>;
 
-    fn array(&mut self, array: &AvroSchema, item: Self::T) -> Result<Self::T>;
-    fn map(&mut self, map: &AvroSchema, value: Self::T) -> Result<Self::T>;
+    fn array(&mut self, array: &ArraySchema, item: Self::T) -> Result<Self::T>;
+    fn map(&mut self, map: &MapSchema, value: Self::T) -> Result<Self::T>;
+    // There are two representation for iceberg map in avro: array of key-value records, or map when keys are strings (optional),
+    // ref: https://iceberg.apache.org/spec/#avro
+    fn map_array(&mut self, array: &RecordSchema, key: Self::T, value: Self::T) -> Result<Self::T>;
 
     fn primitive(&mut self, schema: &AvroSchema) -> Result<Self::T>;
 }
@@ -315,25 +360,73 @@ pub(crate) fn visit<V: AvroSchemaVisitor>(schema: &AvroSchema, visitor: &mut V)
             visitor.union(union, option_results)
         }
         AvroSchema::Array(item) => {
-            let item_result = visit(item, visitor)?;
-            visitor.array(schema, item_result)
+            if let Some(logical_type) = item
+                .attributes
+                .get(LOGICAL_TYPE)
+                .and_then(|v| Value::as_str(v))
+            {
+                if logical_type == MAP_LOGICAL_TYPE {
+                    if let AvroSchema::Record(record_schema) = &*item.items {
+                        let key = visit(&record_schema.fields[0].schema, visitor)?;
+                        let value = visit(&record_schema.fields[1].schema, visitor)?;
+                        return visitor.map_array(record_schema, key, value);
+                    } else {
+                        return Err(Error::new(
+                            ErrorKind::DataInvalid,
+                            "Can't convert avro map schema, item is not a record.",
+                        ));
+                    }
+                } else {
+                    return Err(Error::new(
+                        ErrorKind::FeatureUnsupported,
+                        format!(
+                            "Logical type {logical_type} is not support in iceberg array type.",
+                        ),
+                    ));
+                }
+            }
+            let item_result = visit(&item.items, visitor)?;
+            visitor.array(item, item_result)
         }
         AvroSchema::Map(inner) => {
-            let item_result = visit(inner, visitor)?;
-            visitor.map(schema, item_result)
+            let item_result = visit(&inner.types, visitor)?;
+            visitor.map(inner, item_result)
         }
         schema => visitor.primitive(schema),
     }
 }
 
-struct AvroSchemaToSchema {
-    next_id: i32,
-}
+struct AvroSchemaToSchema;
 
 impl AvroSchemaToSchema {
-    fn next_field_id(&mut self) -> i32 {
-        self.next_id += 1;
-        self.next_id
+    /// A convenient way to get element id(i32) from attributes.
+    #[inline]
+    fn get_element_id_from_attributes(
+        attributes: &BTreeMap<String, Value>,
+        name: &str,
+    ) -> Result<i32> {
+        attributes
+            .get(name)
+            .ok_or_else(|| {
+                Error::new(
+                    ErrorKind::DataInvalid,
+                    "Can't convert avro array schema, missing element id.",
+                )
+            })?
+            .as_i64()
+            .ok_or_else(|| {
+                Error::new(
+                    ErrorKind::DataInvalid,
+                    "Can't convert avro array schema, element id is not a valid i64 number.",
+                )
+            })?
+            .try_into()
+            .map_err(|_| {
+                Error::new(
+                    ErrorKind::DataInvalid,
+                    "Can't convert avro array schema, element id is not a valid i32.",
+                )
+            })
     }
 }
 
@@ -348,24 +441,12 @@ impl AvroSchemaVisitor for AvroSchemaToSchema {
     ) -> Result<Option<Type>> {
         let mut fields = Vec::with_capacity(field_types.len());
         for (avro_field, typ) in record.fields.iter().zip_eq(field_types) {
-            let field_id = avro_field
-                .custom_attributes
-                .get(FILED_ID_PROP)
-                .and_then(Value::as_i64)
-                .ok_or_else(|| {
-                    Error::new(
-                        ErrorKind::DataInvalid,
-                        format!("Can't convert field, missing field id: {avro_field:?}"),
-                    )
-                })?;
+            let field_id =
+                Self::get_element_id_from_attributes(&avro_field.custom_attributes, FILED_ID_PROP)?;
 
             let optional = is_avro_optional(&avro_field.schema);
 
-            let mut field = if optional {
-                NestedField::optional(field_id as i32, &avro_field.name, typ.unwrap())
-            } else {
-                NestedField::required(field_id as i32, &avro_field.name, typ.unwrap())
-            };
+            let mut field = NestedField::new(field_id, &avro_field.name, typ.unwrap(), !optional);
 
             if let Some(doc) = &avro_field.doc {
                 field = field.with_doc(doc);
@@ -403,46 +484,31 @@ impl AvroSchemaVisitor for AvroSchemaToSchema {
         }
     }
 
-    fn array(&mut self, array: &AvroSchema, item: Option<Type>) -> Result<Self::T> {
-        if let AvroSchema::Array(item_schema) = array {
-            let element_field = NestedField::list_element(
-                self.next_field_id(),
-                item.unwrap(),
-                !is_avro_optional(item_schema),
-            )
-            .into();
-            Ok(Some(Type::List(ListType { element_field })))
-        } else {
-            Err(Error::new(
-                ErrorKind::Unexpected,
-                "Expected avro array schema, but {array}",
-            ))
-        }
+    fn array(&mut self, array: &ArraySchema, item: Option<Type>) -> Result<Self::T> {
+        let element_field_id = Self::get_element_id_from_attributes(&array.attributes, ELEMENT_ID)?;
+        let element_field = NestedField::list_element(
+            element_field_id,
+            item.unwrap(),
+            !is_avro_optional(&array.items),
+        )
+        .into();
+        Ok(Some(Type::List(ListType { element_field })))
     }
 
-    fn map(&mut self, map: &AvroSchema, value: Option<Type>) -> Result<Option<Type>> {
-        if let AvroSchema::Map(value_schema) = map {
-            // Due to avro rust implementation's limitation, we can't store attributes in map schema,
-            // we will fix it later when it has been resolved.
-            let key_field = NestedField::map_key_element(
-                self.next_field_id(),
-                Type::Primitive(PrimitiveType::String),
-            );
-            let value_field = NestedField::map_value_element(
-                self.next_field_id(),
-                value.unwrap(),
-                !is_avro_optional(value_schema),
-            );
-            Ok(Some(Type::Map(MapType {
-                key_field: key_field.into(),
-                value_field: value_field.into(),
-            })))
-        } else {
-            Err(Error::new(
-                ErrorKind::Unexpected,
-                "Expected avro map schema, but {map}",
-            ))
-        }
+    fn map(&mut self, map: &MapSchema, value: Option<Type>) -> Result<Option<Type>> {
+        let key_field_id = Self::get_element_id_from_attributes(&map.attributes, KEY_ID)?;
+        let key_field =
+            NestedField::map_key_element(key_field_id, Type::Primitive(PrimitiveType::String));
+        let value_field_id = Self::get_element_id_from_attributes(&map.attributes, VALUE_ID)?;
+        let value_field = NestedField::map_value_element(
+            value_field_id,
+            value.unwrap(),
+            !is_avro_optional(&map.types),
+        );
+        Ok(Some(Type::Map(MapType {
+            key_field: key_field.into(),
+            value_field: value_field.into(),
+        })))
     }
 
     fn primitive(&mut self, schema: &AvroSchema) -> Result<Option<Type>> {
@@ -494,12 +560,53 @@ impl AvroSchemaVisitor for AvroSchemaToSchema {
 
         Ok(Some(typ))
     }
+
+    fn map_array(
+        &mut self,
+        array: &RecordSchema,
+        key: Option<Type>,
+        value: Option<Type>,
+    ) -> Result<Self::T> {
+        let key = key.ok_or_else(|| {
+            Error::new(
+                ErrorKind::DataInvalid,
+                "Can't convert avro map schema, missing key schema.",
+            )
+        })?;
+        let value = value.ok_or_else(|| {
+            Error::new(
+                ErrorKind::DataInvalid,
+                "Can't convert avro map schema, missing value schema.",
+            )
+        })?;
+        let key_id = Self::get_element_id_from_attributes(
+            &array.fields[0].custom_attributes,
+            FILED_ID_PROP,
+        )?;
+        let value_id = Self::get_element_id_from_attributes(
+            &array.fields[1].custom_attributes,
+            FILED_ID_PROP,
+        )?;
+        let key_field = NestedField::map_key_element(key_id, key);
+        let value_field = NestedField::map_value_element(
+            value_id,
+            value,
+            !is_avro_optional(&array.fields[1].schema),
+        );
+        Ok(Some(Type::Map(MapType {
+            key_field: key_field.into(),
+            value_field: value_field.into(),
+        })))
+    }
 }
 
+// # TODO
+// Fix this when we have used `avro_schema_to_schema` inner.
+#[allow(unused)]
 /// Converts avro schema to iceberg schema.
 pub(crate) fn avro_schema_to_schema(avro_schema: &AvroSchema) -> Result<Schema> {
     if let AvroSchema::Record(_) = avro_schema {
-        let mut converter = AvroSchemaToSchema { next_id: 0 };
+        let mut converter = AvroSchemaToSchema;
         let typ = visit(avro_schema, &mut converter)?.expect("Iceberg schema should not be none.");
         if let Type::Struct(s) = typ {
             Schema::builder()
@@ -521,12 +628,15 @@ pub(crate) fn avro_schema_to_schema(avro_schema: &AvroSchema) -> Result<Schema>
 
 #[cfg(test)]
 mod tests {
+    use std::fs::read_to_string;
+    use std::sync::Arc;
+
+    use apache_avro::schema::{Namespace, UnionSchema};
+    use apache_avro::Schema as AvroSchema;
+
     use super::*;
     use crate::avro::schema::AvroSchemaToSchema;
     use crate::spec::{ListType, MapType, NestedField, PrimitiveType, Schema, StructType, Type};
-    use apache_avro::schema::{Namespace, UnionSchema};
-    use apache_avro::Schema as AvroSchema;
-    use std::fs::read_to_string;
 
     fn read_test_data_file_to_avro_schema(filename: &str) -> AvroSchema {
         let input = read_to_string(format!(
@@ -539,22 +649,27 @@ mod tests {
         AvroSchema::parse_str(input.as_str()).unwrap()
     }
 
-    fn check_schema_conversion(
-        avro_schema: AvroSchema,
-        expected_iceberg_schema: Schema,
-        check_avro_to_iceberg: bool,
-    ) {
-        if check_avro_to_iceberg {
-            let converted_iceberg_schema = avro_schema_to_schema(&avro_schema).unwrap();
-            assert_eq!(expected_iceberg_schema, converted_iceberg_schema);
-        }
+    /// Help function to check schema conversion between avro and iceberg:
+    /// 1. avro to iceberg
+    /// 2. iceberg to avro
+    /// 3. iceberg to avro to iceberg back
+    fn check_schema_conversion(avro_schema: AvroSchema, iceberg_schema: Schema) {
+        // 1. avro to iceberg
+        let converted_iceberg_schema = avro_schema_to_schema(&avro_schema).unwrap();
+        assert_eq!(iceberg_schema, converted_iceberg_schema);
 
+        // 2. iceberg to avro
         let converted_avro_schema = schema_to_avro_schema(
             avro_schema.name().unwrap().fullname(Namespace::None),
-            &expected_iceberg_schema,
+            &iceberg_schema,
         )
         .unwrap();
         assert_eq!(avro_schema, converted_avro_schema);
+
+        // 3.iceberg to avro to iceberg back
+        let converted_avro_converted_iceberg_schema =
+            avro_schema_to_schema(&converted_avro_schema).unwrap();
+        assert_eq!(iceberg_schema, converted_avro_converted_iceberg_schema);
     }
 
     #[test]
@@ -633,7 +748,6 @@ mod tests {
         check_schema_conversion(
             read_test_data_file_to_avro_schema("avro_schema_manifest_file_v1.json"),
             iceberg_schema,
-            false,
         );
     }
 
@@ -682,7 +796,7 @@ mod tests {
                 .unwrap()
         };
 
-        check_schema_conversion(avro_schema, iceberg_schema, false);
+        check_schema_conversion(avro_schema, iceberg_schema);
     }
 
     #[test]
@@ -705,7 +819,7 @@ mod tests {
             "field-id": 100
         }
     ]
-}            
+}
 "#,
             )
             .unwrap()
@@ -731,7 +845,7 @@ mod tests {
                 .unwrap()
         };
 
-        check_schema_conversion(avro_schema, iceberg_schema, false);
+        check_schema_conversion(avro_schema, iceberg_schema);
     }
 
     #[test]
@@ -768,7 +882,7 @@ mod tests {
             "field-id": 100
         }
     ]
-}            
+}
 "#,
             )
             .unwrap()
@@ -808,7 +922,144 @@ mod tests {
                 .unwrap()
         };
 
-        check_schema_conversion(avro_schema, iceberg_schema, false);
+        check_schema_conversion(avro_schema, iceberg_schema);
+    }
+
+    #[test]
+    fn test_schema_with_array_map() {
+        let avro_schema = {
+            AvroSchema::parse_str(
+                r#"
+{
+    "type": "record",
+    "name": "avro_schema",
+    "fields": [
+        {
+            "name": "optional",
+            "type": {
+                "type": "array",
+                "items": {
+                    "type": "record",
+                    "name": "k102_v103",
+                    "fields": [
+                        {
+                            "name": "key",
+                            "type": "boolean",
+                            "field-id": 102
+                        },
+                        {
+                            "name": "value",
+                            "type": ["null", "boolean"],
+                            "field-id": 103
+                        }
+                    ]
+                },
+                "default": [],
+                "element-id": 101,
+                "logicalType": "map"
+            },
+            "field-id": 100
+        },{
+            "name": "required",
+            "type": {
+                "type": "array",
+                "items": {
+                    "type": "record",
+                    "name": "k105_v106",
+                    "fields": [
+                        {
+                            "name": "key",
+                            "type": "boolean",
+                            "field-id": 105
+                        },
+                        {
+                            "name": "value",
+                            "type": "boolean",
+                            "field-id": 106
+                        }
+                    ]
+                },
+                "default": [],
+                "logicalType": "map"
+            },
+            "field-id": 104
+        }, {
+            "name": "string_map",
+            "type": {
+                "type": "map",
+                "values": ["null", "long"],
+                "key-id": 108,
+                "value-id": 109
+            },
+            "field-id": 107
+        }
+    ]
+}
+"#,
+            )
+            .unwrap()
+        };
+
+        let iceberg_schema = {
+            Schema::builder()
+                .with_fields(vec![
+                    Arc::new(NestedField::required(
+                        100,
+                        "optional",
+                        Type::Map(MapType {
+                            key_field: NestedField::map_key_element(
+                                102,
+                                PrimitiveType::Boolean.into(),
+                            )
+                            .into(),
+                            value_field: NestedField::map_value_element(
+                                103,
+                                PrimitiveType::Boolean.into(),
+                                false,
+                            )
+                            .into(),
+                        }),
+                    )),
+                    Arc::new(NestedField::required(
+                        104,
+                        "required",
+                        Type::Map(MapType {
+                            key_field: NestedField::map_key_element(
+                                105,
+                                PrimitiveType::Boolean.into(),
+                            )
+                            .into(),
+                            value_field: NestedField::map_value_element(
+                                106,
+                                PrimitiveType::Boolean.into(),
+                                true,
+                            )
+                            .into(),
+                        }),
+                    )),
+                    Arc::new(NestedField::required(
+                        107,
+                        "string_map",
+                        Type::Map(MapType {
+                            key_field: NestedField::map_key_element(
+                                108,
+                                PrimitiveType::String.into(),
+                            )
+                            .into(),
+                            value_field: NestedField::map_value_element(
+                                109,
+                                PrimitiveType::Long.into(),
+                                false,
+                            )
+                            .into(),
+                        }),
+                    )),
+                ])
+                .build()
+                .unwrap()
+        };
+
+        check_schema_conversion(avro_schema, iceberg_schema);
     }
 
     #[test]
@@ -820,7 +1071,7 @@ mod tests {
         ])
         .unwrap();
 
-        let mut converter = AvroSchemaToSchema { next_id: 0 };
+        let mut converter = AvroSchemaToSchema;
 
         let options = avro_schema
             .variants()
@@ -832,7 +1083,7 @@ mod tests {
 
     #[test]
     fn test_string_type() {
-        let mut converter = AvroSchemaToSchema { next_id: 0 };
+        let mut converter = AvroSchemaToSchema;
         let avro_schema = AvroSchema::String;
 
         assert_eq!(
@@ -857,10 +1108,14 @@ mod tests {
             .unwrap()
         };
 
-        let mut converter = AvroSchemaToSchema { next_id: 0 };
+        let AvroSchema::Map(avro_schema) = avro_schema else {
+            unreachable!()
+        };
+
+        let mut converter = AvroSchemaToSchema;
         let iceberg_type = Type::Map(MapType {
-            key_field: NestedField::map_key_element(1, PrimitiveType::String.into()).into(),
-            value_field: NestedField::map_value_element(2, PrimitiveType::Long.into(), false)
+            key_field: NestedField::map_key_element(101, PrimitiveType::String.into()).into(),
+            value_field: NestedField::map_value_element(102, PrimitiveType::Long.into(), false)
                 .into(),
         });
 
@@ -884,7 +1139,7 @@ mod tests {
             .unwrap()
         };
 
-        let mut converter = AvroSchemaToSchema { next_id: 0 };
+        let mut converter = AvroSchemaToSchema;
 
         let iceberg_type = Type::from(PrimitiveType::Fixed(22));
 
@@ -896,7 +1151,7 @@ mod tests {
 
     #[test]
     fn test_unknown_primitive() {
-        let mut converter = AvroSchemaToSchema { next_id: 0 };
+        let mut converter = AvroSchemaToSchema;
 
         assert!(converter.primitive(&AvroSchema::Duration).is_err());
     }
@@ -915,7 +1170,7 @@ mod tests {
             "type": "string"
         }
     ]
-}               
+}
 "#,
             )
             .unwrap()
@@ -935,7 +1190,7 @@ mod tests {
             .unwrap()
         };
 
-        let mut converter = AvroSchemaToSchema { next_id: 0 };
+        let mut converter = AvroSchemaToSchema;
 
         assert_eq!(
             Type::decimal(25, 19).unwrap(),
@@ -945,7 +1200,7 @@ mod tests {
 
     #[test]
     fn test_date_type() {
-        let mut converter = AvroSchemaToSchema { next_id: 0 };
+        let mut converter = AvroSchemaToSchema;
 
         assert_eq!(
             Type::from(PrimitiveType::Date),
