@@ -17,25 +17,27 @@
 
 //! Catalog API for Apache Iceberg
 
-use serde_derive::{Deserialize, Serialize};
-use urlencoding::encode;
-
-use crate::spec::{
-    FormatVersion, Schema, Snapshot, SnapshotReference, SortOrder, UnboundPartitionSpec,
-};
-use crate::table::Table;
-use crate::{Error, ErrorKind, Result};
-use async_trait::async_trait;
 use std::collections::HashMap;
+use std::fmt::Debug;
 use std::mem::take;
 use std::ops::Deref;
+
+use async_trait::async_trait;
+use serde_derive::{Deserialize, Serialize};
 use typed_builder::TypedBuilder;
 use uuid::Uuid;
 
+use crate::spec::{
+    FormatVersion, Schema, Snapshot, SnapshotReference, SortOrder, TableMetadataBuilder,
+    UnboundPartitionSpec, ViewRepresentations,
+};
+use crate::table::Table;
+use crate::{Error, ErrorKind, Result};
+
 /// The catalog API for Iceberg Rust.
 #[async_trait]
-pub trait Catalog: std::fmt::Debug {
-    /// List namespaces from table.
+pub trait Catalog: Debug + Sync + Send {
+    /// List namespaces inside the catalog.
     async fn list_namespaces(&self, parent: Option<&NamespaceIdent>)
         -> Result<Vec<NamespaceIdent>>;
 
@@ -50,7 +52,7 @@ pub trait Catalog: std::fmt::Debug {
     async fn get_namespace(&self, namespace: &NamespaceIdent) -> Result<Namespace>;
 
     /// Check if namespace exists in catalog.
-    async fn namespace_exists(&self, namesace: &NamespaceIdent) -> Result<bool>;
+    async fn namespace_exists(&self, namespace: &NamespaceIdent) -> Result<bool>;
 
     /// Update a namespace inside the catalog.
     ///
@@ -83,7 +85,7 @@ pub trait Catalog: std::fmt::Debug {
     async fn drop_table(&self, table: &TableIdent) -> Result<()>;
 
     /// Check if a table exists in the catalog.
-    async fn stat_table(&self, table: &TableIdent) -> Result<bool>;
+    async fn table_exists(&self, table: &TableIdent) -> Result<bool>;
 
     /// Rename a table in the catalog.
     async fn rename_table(&self, src: &TableIdent, dest: &TableIdent) -> Result<()>;
@@ -122,9 +124,9 @@ impl NamespaceIdent {
         Self::from_vec(iter.into_iter().map(|s| s.to_string()).collect())
     }
 
-    /// Returns url encoded format.
-    pub fn encode_in_url(&self) -> String {
-        encode(&self.as_ref().join("\u{1F}")).to_string()
+    /// Returns a string for used in url.
+    pub fn to_url_string(&self) -> String {
+        self.as_ref().join("\u{001f}")
     }
 
     /// Returns inner strings.
@@ -227,7 +229,7 @@ pub struct TableCreation {
     /// The schema of the table.
     pub schema: Schema,
     /// The partition spec of the table, could be None.
-    #[builder(default, setter(strip_option))]
+    #[builder(default, setter(strip_option, into))]
     pub partition_spec: Option<UnboundPartitionSpec>,
     /// The sort order of the table.
     #[builder(default, setter(strip_option))]
@@ -331,7 +333,7 @@ pub enum TableRequirement {
 }
 
 /// TableUpdate represents an update to a table in the catalog.
-#[derive(Debug, Serialize, Deserialize, PartialEq)]
+#[derive(Debug, Serialize, Deserialize, PartialEq, Clone)]
 #[serde(tag = "action", rename_all = "kebab-case")]
 pub enum TableUpdate {
     /// Upgrade table's format version
@@ -381,7 +383,7 @@ pub enum TableUpdate {
     #[serde(rename_all = "kebab-case")]
     SetDefaultSortOrder {
         /// Sort order ID to set as the default, or -1 to set last added sort order
-        sort_order_id: i32,
+        sort_order_id: i64,
     },
     /// Add snapshot to table.
     #[serde(rename_all = "kebab-case")]
@@ -427,20 +429,56 @@ pub enum TableUpdate {
     },
 }
 
+impl TableUpdate {
+    /// Applies the update to the table metadata builder.
+    pub fn apply(self, builder: TableMetadataBuilder) -> Result<TableMetadataBuilder> {
+        match self {
+            TableUpdate::AssignUuid { uuid } => builder.assign_uuid(uuid),
+            _ => unimplemented!(),
+        }
+    }
+}
+
+/// ViewCreation represents the creation of a view in the catalog.
+#[derive(Debug, TypedBuilder)]
+pub struct ViewCreation {
+    /// The name of the view.
+    pub name: String,
+    /// The view's base location; used to create metadata file locations
+    pub location: String,
+    /// Representations for the view.
+    pub representations: ViewRepresentations,
+    /// The schema of the view.
+    pub schema: Schema,
+    /// The properties of the view.
+    #[builder(default)]
+    pub properties: HashMap<String, String>,
+    /// The default namespace to use when a reference in the SELECT is a single identifier
+    pub default_namespace: NamespaceIdent,
+    /// Default catalog to use when a reference in the SELECT does not contain a catalog
+    #[builder(default)]
+    pub default_catalog: Option<String>,
+    /// A string to string map of summary metadata about the version
+    /// Typical keys are "engine-name" and "engine-version"
+    #[builder(default)]
+    pub summary: HashMap<String, String>,
+}
+
 #[cfg(test)]
 mod tests {
-    use crate::spec::ManifestListLocation::ManifestListFile;
+    use std::collections::HashMap;
+    use std::fmt::Debug;
+
+    use serde::de::DeserializeOwned;
+    use serde::Serialize;
+    use uuid::uuid;
+
     use crate::spec::{
         FormatVersion, NestedField, NullOrder, Operation, PrimitiveType, Schema, Snapshot,
         SnapshotReference, SnapshotRetention, SortDirection, SortField, SortOrder, Summary,
-        Transform, Type, UnboundPartitionField, UnboundPartitionSpec,
+        TableMetadataBuilder, Transform, Type, UnboundPartitionSpec,
     };
-    use crate::{NamespaceIdent, TableIdent, TableRequirement, TableUpdate};
-    use serde::de::DeserializeOwned;
-    use serde::Serialize;
-    use std::collections::HashMap;
-    use std::fmt::Debug;
-    use uuid::uuid;
+    use crate::{NamespaceIdent, TableCreation, TableIdent, TableRequirement, TableUpdate};
 
     #[test]
     fn test_create_table_id() {
@@ -782,29 +820,13 @@ mod tests {
         "#,
             TableUpdate::AddSpec {
                 spec: UnboundPartitionSpec::builder()
-                    .with_unbound_partition_field(
-                        UnboundPartitionField::builder()
-                            .source_id(4)
-                            .name("ts_day".to_string())
-                            .transform(Transform::Day)
-                            .build(),
-                    )
-                    .with_unbound_partition_field(
-                        UnboundPartitionField::builder()
-                            .source_id(1)
-                            .name("id_bucket".to_string())
-                            .transform(Transform::Bucket(16))
-                            .build(),
-                    )
-                    .with_unbound_partition_field(
-                        UnboundPartitionField::builder()
-                            .source_id(2)
-                            .name("id_truncate".to_string())
-                            .transform(Transform::Truncate(4))
-                            .build(),
-                    )
-                    .build()
-                    .unwrap(),
+                    .add_partition_field(4, "ts_day".to_string(), Transform::Day)
+                    .unwrap()
+                    .add_partition_field(1, "id_bucket".to_string(), Transform::Bucket(16))
+                    .unwrap()
+                    .add_partition_field(2, "id_truncate".to_string(), Transform::Truncate(4))
+                    .unwrap()
+                    .build(),
             },
         );
     }
@@ -866,7 +888,7 @@ mod tests {
                         .transform(Transform::Bucket(4))
                         .build(),
                 )
-                .build()
+                .build_unbound()
                 .unwrap(),
         };
 
@@ -911,7 +933,7 @@ mod tests {
                 .with_parent_snapshot_id(Some(3051729675574597000))
                 .with_timestamp_ms(1555100955770)
                 .with_sequence_number(1)
-                .with_manifest_list(ManifestListFile("s3://a/b/2.avro".to_string()))
+                .with_manifest_list("s3://a/b/2.avro")
                 .with_schema_id(1)
                 .with_summary(Summary {
                     operation: Operation::Append,
@@ -972,7 +994,9 @@ mod tests {
             ref_name: "hank".to_string(),
             reference: SnapshotReference {
                 snapshot_id: 1,
-                retention: SnapshotRetention::Tag { max_ref_age_ms: 1 },
+                retention: SnapshotRetention::Tag {
+                    max_ref_age_ms: Some(1),
+                },
             },
         };
 
@@ -1066,4 +1090,28 @@ mod tests {
 
         test_serde_json(json, update);
     }
+
+    #[test]
+    fn test_table_update_apply() {
+        let table_creation = TableCreation::builder()
+            .location("s3://db/table".to_string())
+            .name("table".to_string())
+            .properties(HashMap::new())
+            .schema(Schema::builder().build().unwrap())
+            .build();
+        let table_metadata = TableMetadataBuilder::from_table_creation(table_creation)
+            .unwrap()
+            .build()
+            .unwrap();
+        let table_metadata_builder = TableMetadataBuilder::new(table_metadata);
+
+        let uuid = uuid::Uuid::new_v4();
+        let update = TableUpdate::AssignUuid { uuid };
+        let updated_metadata = update
+            .apply(table_metadata_builder)
+            .unwrap()
+            .build()
+            .unwrap();
+        assert_eq!(updated_metadata.uuid(), uuid);
+    }
 }
