@@ -17,20 +17,23 @@
 
 /*!
  * Data Types
-*/
-use crate::ensure_data_valid;
-use crate::error::Result;
-use crate::spec::datatypes::_decimal::{MAX_PRECISION, REQUIRED_LENGTH};
+ */
+use std::collections::HashMap;
+use std::convert::identity;
+use std::fmt;
+use std::ops::Index;
+use std::sync::{Arc, OnceLock};
+
 use ::serde::de::{MapAccess, Visitor};
 use serde::de::{Error, IntoDeserializer};
 use serde::{de, Deserialize, Deserializer, Serialize, Serializer};
 use serde_json::Value as JsonValue;
-use std::convert::identity;
-use std::sync::Arc;
-use std::sync::OnceLock;
-use std::{collections::HashMap, fmt, ops::Index};
 
 use super::values::Literal;
+use crate::ensure_data_valid;
+use crate::error::Result;
+use crate::spec::datatypes::_decimal::{MAX_PRECISION, REQUIRED_LENGTH};
+use crate::spec::PrimitiveLiteral;
 
 /// Field name for list type.
 pub(crate) const LIST_FILED_NAME: &str = "element";
@@ -41,38 +44,37 @@ pub(crate) const MAX_DECIMAL_BYTES: u32 = 24;
 pub(crate) const MAX_DECIMAL_PRECISION: u32 = 38;
 
 mod _decimal {
-    use lazy_static::lazy_static;
+    use once_cell::sync::Lazy;
 
     use crate::spec::{MAX_DECIMAL_BYTES, MAX_DECIMAL_PRECISION};
 
-    lazy_static! {
-        // Max precision of bytes, starts from 1
-        pub(super) static ref MAX_PRECISION: [u32; MAX_DECIMAL_BYTES as usize] = {
-            let mut ret: [u32; 24] = [0; 24];
-            for (i, prec) in ret.iter_mut().enumerate() {
-                *prec = 2f64.powi((8 * (i + 1) - 1) as i32).log10().floor() as u32;
-            }
+    // Max precision of bytes, starts from 1
+    pub(super) static MAX_PRECISION: Lazy<[u32; MAX_DECIMAL_BYTES as usize]> = Lazy::new(|| {
+        let mut ret: [u32; 24] = [0; 24];
+        for (i, prec) in ret.iter_mut().enumerate() {
+            *prec = 2f64.powi((8 * (i + 1) - 1) as i32).log10().floor() as u32;
+        }
 
-            ret
-        };
+        ret
+    });
 
-        //  Required bytes of precision, starts from 1
-        pub(super) static ref REQUIRED_LENGTH: [u32; MAX_DECIMAL_PRECISION as usize] = {
-            let mut ret: [u32; MAX_DECIMAL_PRECISION as usize] = [0; MAX_DECIMAL_PRECISION as usize];
+    //  Required bytes of precision, starts from 1
+    pub(super) static REQUIRED_LENGTH: Lazy<[u32; MAX_DECIMAL_PRECISION as usize]> =
+        Lazy::new(|| {
+            let mut ret: [u32; MAX_DECIMAL_PRECISION as usize] =
+                [0; MAX_DECIMAL_PRECISION as usize];
 
             for (i, required_len) in ret.iter_mut().enumerate() {
                 for j in 0..MAX_PRECISION.len() {
-                    if MAX_PRECISION[j] >= ((i+1) as u32) {
-                        *required_len = (j+1) as u32;
+                    if MAX_PRECISION[j] >= ((i + 1) as u32) {
+                        *required_len = (j + 1) as u32;
                         break;
                     }
                 }
             }
 
             ret
-        };
-
-    }
+        });
 }
 
 #[derive(Debug, PartialEq, Eq, Clone)]
@@ -112,6 +114,30 @@ impl Type {
         matches!(self, Type::Struct(_))
     }
 
+    /// Whether the type is nested type.
+    #[inline(always)]
+    pub fn is_nested(&self) -> bool {
+        matches!(self, Type::Struct(_) | Type::List(_) | Type::Map(_))
+    }
+
+    /// Convert Type to reference of PrimitiveType
+    pub fn as_primitive_type(&self) -> Option<&PrimitiveType> {
+        if let Type::Primitive(primitive_type) = self {
+            Some(primitive_type)
+        } else {
+            None
+        }
+    }
+
+    /// Convert Type to StructType
+    pub fn to_struct_type(self) -> Option<StructType> {
+        if let Type::Struct(struct_type) = self {
+            Some(struct_type)
+        } else {
+            None
+        }
+    }
+
     /// Return max precision for decimal given [`num_bytes`] bytes.
     #[inline(always)]
     pub fn decimal_max_precision(num_bytes: u32) -> Result<u32> {
@@ -135,6 +161,15 @@ impl Type {
         ensure_data_valid!(precision > 0 && precision <= MAX_DECIMAL_PRECISION, "Decimals with precision larger than {MAX_DECIMAL_PRECISION} are not supported: {precision}",);
         Ok(Type::Primitive(PrimitiveType::Decimal { precision, scale }))
     }
+
+    /// Check if it's float or double type.
+    #[inline(always)]
+    pub fn is_floating_type(&self) -> bool {
+        matches!(
+            self,
+            Type::Primitive(PrimitiveType::Float) | Type::Primitive(PrimitiveType::Double)
+        )
+    }
 }
 
 impl From<PrimitiveType> for Type {
@@ -162,7 +197,7 @@ impl From<MapType> for Type {
 }
 
 /// Primitive data types
-#[derive(Debug, Serialize, Deserialize, PartialEq, Eq, Clone)]
+#[derive(Debug, Serialize, Deserialize, PartialEq, Eq, Clone, Hash)]
 #[serde(rename_all = "lowercase", remote = "Self")]
 pub enum PrimitiveType {
     /// True or False
@@ -171,28 +206,28 @@ pub enum PrimitiveType {
     Int,
     /// 64-bit signed integer
     Long,
-    /// 32-bit IEEE 754 floating bit.
+    /// 32-bit IEEE 754 floating point.
     Float,
-    /// 64-bit IEEE 754 floating bit.
+    /// 64-bit IEEE 754 floating point.
     Double,
     /// Fixed point decimal
     Decimal {
-        /// Precision
+        /// Precision, must be 38 or less
         precision: u32,
         /// Scale
         scale: u32,
     },
     /// Calendar date without timezone or time.
     Date,
-    /// Time of day without date or timezone.
+    /// Time of day in microsecond precision, without date or timezone.
     Time,
-    /// Timestamp without timezone
+    /// Timestamp in microsecond precision, without timezone
     Timestamp,
-    /// Timestamp with timezone
+    /// Timestamp in microsecond precision, with timezone
     Timestamptz,
     /// Arbitrary-length character sequences encoded in utf-8
     String,
-    /// Universally Unique Identifiers
+    /// Universally Unique Identifiers, should use 16-byte fixed
     Uuid,
     /// Fixed length byte array
     Fixed(u64),
@@ -200,11 +235,32 @@ pub enum PrimitiveType {
     Binary,
 }
 
+impl PrimitiveType {
+    /// Check whether literal is compatible with the type.
+    pub fn compatible(&self, literal: &PrimitiveLiteral) -> bool {
+        matches!(
+            (self, literal),
+            (PrimitiveType::Boolean, PrimitiveLiteral::Boolean(_))
+                | (PrimitiveType::Int, PrimitiveLiteral::Int(_))
+                | (PrimitiveType::Long, PrimitiveLiteral::Long(_))
+                | (PrimitiveType::Float, PrimitiveLiteral::Float(_))
+                | (PrimitiveType::Double, PrimitiveLiteral::Double(_))
+                | (PrimitiveType::Decimal { .. }, PrimitiveLiteral::Int128(_))
+                | (PrimitiveType::Date, PrimitiveLiteral::Int(_))
+                | (PrimitiveType::Time, PrimitiveLiteral::Long(_))
+                | (PrimitiveType::Timestamp, PrimitiveLiteral::Long(_))
+                | (PrimitiveType::Timestamptz, PrimitiveLiteral::Long(_))
+                | (PrimitiveType::String, PrimitiveLiteral::String(_))
+                | (PrimitiveType::Uuid, PrimitiveLiteral::UInt128(_))
+                | (PrimitiveType::Fixed(_), PrimitiveLiteral::Binary(_))
+                | (PrimitiveType::Binary, PrimitiveLiteral::Binary(_))
+        )
+    }
+}
+
 impl Serialize for Type {
     fn serialize<S>(&self, serializer: S) -> std::result::Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
+    where S: Serializer {
         let type_serde = _serde::SerdeType::from(self);
         type_serde.serialize(serializer)
     }
@@ -212,9 +268,7 @@ impl Serialize for Type {
 
 impl<'de> Deserialize<'de> for Type {
     fn deserialize<D>(deserializer: D) -> std::result::Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
+    where D: Deserializer<'de> {
         let type_serde = _serde::SerdeType::deserialize(deserializer)?;
         Ok(Type::from(type_serde))
     }
@@ -222,9 +276,7 @@ impl<'de> Deserialize<'de> for Type {
 
 impl<'de> Deserialize<'de> for PrimitiveType {
     fn deserialize<D>(deserializer: D) -> std::result::Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
+    where D: Deserializer<'de> {
         let s = String::deserialize(deserializer)?;
         if s.starts_with("decimal") {
             deserialize_decimal(s.into_deserializer())
@@ -238,9 +290,7 @@ impl<'de> Deserialize<'de> for PrimitiveType {
 
 impl Serialize for PrimitiveType {
     fn serialize<S>(&self, serializer: S) -> std::result::Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
+    where S: Serializer {
         match self {
             PrimitiveType::Decimal { precision, scale } => {
                 serialize_decimal(precision, scale, serializer)
@@ -252,9 +302,7 @@ impl Serialize for PrimitiveType {
 }
 
 fn deserialize_decimal<'de, D>(deserializer: D) -> std::result::Result<PrimitiveType, D::Error>
-where
-    D: Deserializer<'de>,
-{
+where D: Deserializer<'de> {
     let s = String::deserialize(deserializer)?;
     let (precision, scale) = s
         .trim_start_matches(r"decimal(")
@@ -280,9 +328,7 @@ where
 }
 
 fn deserialize_fixed<'de, D>(deserializer: D) -> std::result::Result<PrimitiveType, D::Error>
-where
-    D: Deserializer<'de>,
-{
+where D: Deserializer<'de> {
     let fixed = String::deserialize(deserializer)?
         .trim_start_matches(r"fixed[")
         .trim_end_matches(']')
@@ -295,9 +341,7 @@ where
 }
 
 fn serialize_fixed<S>(value: &u64, serializer: S) -> std::result::Result<S::Ok, S::Error>
-where
-    S: Serializer,
-{
+where S: Serializer {
     serializer.serialize_str(&format!("fixed[{value}]"))
 }
 
@@ -325,7 +369,7 @@ impl fmt::Display for PrimitiveType {
 }
 
 /// DataType for a specific struct
-#[derive(Debug, Serialize, Clone)]
+#[derive(Debug, Serialize, Clone, Default)]
 #[serde(rename = "struct", tag = "type")]
 pub struct StructType {
     /// Struct fields
@@ -339,9 +383,7 @@ pub struct StructType {
 
 impl<'de> Deserialize<'de> for StructType {
     fn deserialize<D>(deserializer: D) -> std::result::Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
+    where D: Deserializer<'de> {
         #[derive(Deserialize)]
         #[serde(field_identifier, rename_all = "lowercase")]
         enum Field {
@@ -359,9 +401,7 @@ impl<'de> Deserialize<'de> for StructType {
             }
 
             fn visit_map<V>(self, mut map: V) -> std::result::Result<StructType, V::Error>
-            where
-                V: MapAccess<'de>,
-            {
+            where V: MapAccess<'de> {
                 let mut fields = None;
                 while let Some(key) = map.next_key()? {
                     match key {
@@ -541,6 +581,19 @@ impl From<NestedField> for SerdeNestedField {
 pub type NestedFieldRef = Arc<NestedField>;
 
 impl NestedField {
+    /// Construct a new field.
+    pub fn new(id: i32, name: impl ToString, field_type: Type, required: bool) -> Self {
+        Self {
+            id,
+            name: name.to_string(),
+            required,
+            field_type: Box::new(field_type),
+            doc: None,
+            initial_default: None,
+            write_default: None,
+        }
+    }
+
     /// Construct a required field.
     pub fn required(id: i32, name: impl ToString, field_type: Type) -> Self {
         Self {
@@ -634,14 +687,23 @@ pub struct ListType {
     pub element_field: NestedFieldRef,
 }
 
+impl ListType {
+    /// Construct a list type with the given element field.
+    pub fn new(element_field: NestedFieldRef) -> Self {
+        Self { element_field }
+    }
+}
+
 /// Module for type serialization/deserialization.
 pub(super) mod _serde {
+    use std::borrow::Cow;
+
+    use serde_derive::{Deserialize, Serialize};
+
     use crate::spec::datatypes::Type::Map;
     use crate::spec::datatypes::{
         ListType, MapType, NestedField, NestedFieldRef, PrimitiveType, StructType, Type,
     };
-    use serde_derive::{Deserialize, Serialize};
-    use std::borrow::Cow;
 
     /// List type for serialization and deserialization
     #[derive(Serialize, Deserialize)]
@@ -749,14 +811,23 @@ pub struct MapType {
     pub value_field: NestedFieldRef,
 }
 
+impl MapType {
+    /// Construct a map type with the given key and value fields.
+    pub fn new(key_field: NestedFieldRef, value_field: NestedFieldRef) -> Self {
+        Self {
+            key_field,
+            value_field,
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use pretty_assertions::assert_eq;
     use uuid::Uuid;
 
-    use crate::spec::values::PrimitiveLiteral;
-
     use super::*;
+    use crate::spec::values::PrimitiveLiteral;
 
     fn check_type_serde(json: &str, expected_type: Type) {
         let desered_type: Type = serde_json::from_str(json).unwrap();
@@ -862,11 +933,15 @@ mod tests {
             Type::Struct(StructType {
                 fields: vec![
                     NestedField::required(1, "id", Type::Primitive(PrimitiveType::Uuid))
-                        .with_initial_default(Literal::Primitive(PrimitiveLiteral::UUID(
-                            Uuid::parse_str("0db3e2a8-9d1d-42b9-aa7b-74ebe558dceb").unwrap(),
+                        .with_initial_default(Literal::Primitive(PrimitiveLiteral::UInt128(
+                            Uuid::parse_str("0db3e2a8-9d1d-42b9-aa7b-74ebe558dceb")
+                                .unwrap()
+                                .as_u128(),
                         )))
-                        .with_write_default(Literal::Primitive(PrimitiveLiteral::UUID(
-                            Uuid::parse_str("ec5911be-b0a7-458c-8438-c9a3e53cffae").unwrap(),
+                        .with_write_default(Literal::Primitive(PrimitiveLiteral::UInt128(
+                            Uuid::parse_str("ec5911be-b0a7-458c-8438-c9a3e53cffae")
+                                .unwrap()
+                                .as_u128(),
                         )))
                         .into(),
                     NestedField::optional(2, "data", Type::Primitive(PrimitiveType::Int)).into(),
@@ -931,11 +1006,15 @@ mod tests {
 
         let struct_type = Type::Struct(StructType::new(vec![
             NestedField::required(1, "id", Type::Primitive(PrimitiveType::Uuid))
-                .with_initial_default(Literal::Primitive(PrimitiveLiteral::UUID(
-                    Uuid::parse_str("0db3e2a8-9d1d-42b9-aa7b-74ebe558dceb").unwrap(),
+                .with_initial_default(Literal::Primitive(PrimitiveLiteral::UInt128(
+                    Uuid::parse_str("0db3e2a8-9d1d-42b9-aa7b-74ebe558dceb")
+                        .unwrap()
+                        .as_u128(),
                 )))
-                .with_write_default(Literal::Primitive(PrimitiveLiteral::UUID(
-                    Uuid::parse_str("ec5911be-b0a7-458c-8438-c9a3e53cffae").unwrap(),
+                .with_write_default(Literal::Primitive(PrimitiveLiteral::UInt128(
+                    Uuid::parse_str("ec5911be-b0a7-458c-8438-c9a3e53cffae")
+                        .unwrap()
+                        .as_u128(),
                 )))
                 .into(),
             NestedField::optional(2, "data", Type::Primitive(PrimitiveType::Int)).into(),
@@ -1053,4 +1132,35 @@ mod tests {
         assert_eq!(5, Type::decimal_required_bytes(10).unwrap());
         assert_eq!(16, Type::decimal_required_bytes(38).unwrap());
     }
+
+    #[test]
+    fn test_primitive_type_compatitable() {
+        let pairs = vec![
+            (PrimitiveType::Boolean, PrimitiveLiteral::Boolean(true)),
+            (PrimitiveType::Int, PrimitiveLiteral::Int(1)),
+            (PrimitiveType::Long, PrimitiveLiteral::Long(1)),
+            (PrimitiveType::Float, PrimitiveLiteral::Float(1.0.into())),
+            (PrimitiveType::Double, PrimitiveLiteral::Double(1.0.into())),
+            (
+                PrimitiveType::Decimal {
+                    precision: 9,
+                    scale: 2,
+                },
+                PrimitiveLiteral::Int128(1),
+            ),
+            (PrimitiveType::Date, PrimitiveLiteral::Int(1)),
+            (PrimitiveType::Time, PrimitiveLiteral::Long(1)),
+            (PrimitiveType::Timestamptz, PrimitiveLiteral::Long(1)),
+            (PrimitiveType::Timestamp, PrimitiveLiteral::Long(1)),
+            (
+                PrimitiveType::Uuid,
+                PrimitiveLiteral::UInt128(Uuid::new_v4().as_u128()),
+            ),
+            (PrimitiveType::Fixed(8), PrimitiveLiteral::Binary(vec![1])),
+            (PrimitiveType::Binary, PrimitiveLiteral::Binary(vec![1])),
+        ];
+        for (ty, literal) in pairs {
+            assert!(ty.compatible(&literal));
+        }
+    }
 }
