@@ -16,26 +16,26 @@
 // under the License.
 
 //! Manifest for Iceberg.
-use self::_const_schema::{manifest_schema_v1, manifest_schema_v2};
-
-use super::{
-    FieldSummary, FormatVersion, ManifestContentType, ManifestListEntry, PartitionSpec, Schema,
-    SchemaId, Struct, INITIAL_SEQUENCE_NUMBER,
-};
-use super::{Literal, UNASSIGNED_SEQUENCE_NUMBER};
-use crate::error::Result;
-use crate::io::OutputFile;
-use crate::spec::PartitionField;
-use crate::{Error, ErrorKind};
-use apache_avro::{from_value, to_value, Reader as AvroReader, Writer as AvroWriter};
-use futures::AsyncWriteExt;
-use serde_json::to_vec;
 use std::cmp::min;
 use std::collections::HashMap;
 use std::str::FromStr;
 use std::sync::Arc;
+
+use apache_avro::{from_value, to_value, Reader as AvroReader, Writer as AvroWriter};
+use bytes::Bytes;
+use serde_json::to_vec;
 use typed_builder::TypedBuilder;
 
+use self::_const_schema::{manifest_schema_v1, manifest_schema_v2};
+use super::{
+    Datum, FieldSummary, FormatVersion, ManifestContentType, ManifestFile, PartitionSpec, Schema,
+    SchemaId, Struct, INITIAL_SEQUENCE_NUMBER, UNASSIGNED_SEQUENCE_NUMBER,
+};
+use crate::error::Result;
+use crate::io::OutputFile;
+use crate::spec::PartitionField;
+use crate::{Error, ErrorKind};
+
 /// A manifest contains metadata and a list of entries.
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub struct Manifest {
@@ -94,6 +94,12 @@ impl Manifest {
         &self.entries
     }
 
+    /// Consume this Manifest, returning its constituent parts
+    pub fn consume(self) -> (Vec<ManifestEntryRef>, ManifestMetadata) {
+        let Self { entries, metadata } = self;
+        (entries, metadata)
+    }
+
     /// Constructor from [`ManifestMetadata`] and [`ManifestEntry`]s.
     pub fn new(metadata: ManifestMetadata, entries: Vec<ManifestEntry>) -> Self {
         Self {
@@ -195,8 +201,8 @@ impl ManifestWriter {
         partition_summary
     }
 
-    /// Write a manifest entry.
-    pub async fn write(mut self, manifest: Manifest) -> Result<ManifestListEntry> {
+    /// Write a manifest.
+    pub async fn write(mut self, manifest: Manifest) -> Result<ManifestFile> {
         // Create the avro writer
         let partition_type = manifest
             .metadata
@@ -221,14 +227,14 @@ impl ManifestWriter {
         )?;
         avro_writer.add_user_metadata(
             "partition-spec".to_string(),
-            to_vec(&manifest.metadata.partition_spec.fields).map_err(|err| {
+            to_vec(&manifest.metadata.partition_spec.fields()).map_err(|err| {
                 Error::new(ErrorKind::DataInvalid, "Fail to serialize partition spec")
                     .with_source(err)
             })?,
         )?;
         avro_writer.add_user_metadata(
             "partition-spec-id".to_string(),
-            manifest.metadata.partition_spec.spec_id.to_string(),
+            manifest.metadata.partition_spec.spec_id().to_string(),
         )?;
         avro_writer.add_user_metadata(
             "format-version".to_string(),
@@ -289,32 +295,26 @@ impl ManifestWriter {
             avro_writer.append(value)?;
         }
 
-        let length = avro_writer.flush()?;
         let content = avro_writer.into_inner()?;
-        let mut writer = self.output.writer().await?;
-        writer.write_all(&content).await.map_err(|err| {
-            Error::new(ErrorKind::Unexpected, "Fail to write Manifest Entry").with_source(err)
-        })?;
-        writer.close().await.map_err(|err| {
-            Error::new(ErrorKind::Unexpected, "Fail to write Manifest Entry").with_source(err)
-        })?;
+        let length = content.len();
+        self.output.write(Bytes::from(content)).await?;
 
         let partition_summary =
-            self.get_field_summary_vec(&manifest.metadata.partition_spec.fields);
+            self.get_field_summary_vec(manifest.metadata.partition_spec.fields());
 
-        Ok(ManifestListEntry {
+        Ok(ManifestFile {
             manifest_path: self.output.location().to_string(),
             manifest_length: length as i64,
-            partition_spec_id: manifest.metadata.partition_spec.spec_id,
+            partition_spec_id: manifest.metadata.partition_spec.spec_id(),
             content: manifest.metadata.content,
             // sequence_number and min_sequence_number with UNASSIGNED_SEQUENCE_NUMBER will be replace with
             // real sequence number in `ManifestListWriter`.
             sequence_number: UNASSIGNED_SEQUENCE_NUMBER,
             min_sequence_number: self.min_seq_num.unwrap_or(UNASSIGNED_SEQUENCE_NUMBER),
             added_snapshot_id: self.snapshot_id,
-            added_data_files_count: Some(self.added_files),
-            existing_data_files_count: Some(self.existing_files),
-            deleted_data_files_count: Some(self.deleted_files),
+            added_files_count: Some(self.added_files),
+            existing_files_count: Some(self.existing_files),
+            deleted_files_count: Some(self.deleted_files),
             added_rows_count: Some(self.added_rows),
             existing_rows_count: Some(self.existing_rows),
             deleted_rows_count: Some(self.deleted_rows),
@@ -331,13 +331,11 @@ mod _const_schema {
     use apache_avro::Schema as AvroSchema;
     use once_cell::sync::Lazy;
 
-    use crate::{
-        avro::schema_to_avro_schema,
-        spec::{
-            ListType, MapType, NestedField, NestedFieldRef, PrimitiveType, Schema, StructType, Type,
-        },
-        Error,
+    use crate::avro::schema_to_avro_schema;
+    use crate::spec::{
+        ListType, MapType, NestedField, NestedFieldRef, PrimitiveType, Schema, StructType, Type,
     };
+    use crate::Error;
 
     static STATUS: Lazy<NestedFieldRef> = {
         Lazy::new(|| {
@@ -695,7 +693,7 @@ mod _const_schema {
                 ])),
             )),
         ];
-        let schema = Schema::builder().with_fields(fields).build().unwrap();
+        let schema = Schema::builder().with_fields(fields).build()?;
         schema_to_avro_schema("manifest_entry", &schema)
     }
 }
@@ -851,17 +849,25 @@ impl ManifestEntry {
     }
 
     /// Content type of this manifest entry.
+    #[inline]
     pub fn content_type(&self) -> DataContentType {
         self.data_file.content
     }
 
+    /// File format of this manifest entry.
+    #[inline]
+    pub fn file_format(&self) -> DataFileFormat {
+        self.data_file.file_format
+    }
+
     /// Data file path of this manifest entry.
+    #[inline]
     pub fn file_path(&self) -> &str {
         &self.data_file.file_path
     }
 
     /// Inherit data from manifest list, such as snapshot id, sequence number.
-    pub(crate) fn inherit_data(&mut self, snapshot_entry: &ManifestListEntry) {
+    pub(crate) fn inherit_data(&mut self, snapshot_entry: &ManifestFile) {
         if self.snapshot_id.is_none() {
             self.snapshot_id = Some(snapshot_entry.added_snapshot_id);
         }
@@ -892,6 +898,12 @@ impl ManifestEntry {
     pub fn file_size_in_bytes(&self) -> u64 {
         self.data_file.file_size_in_bytes
     }
+
+    /// get a reference to the actual data file
+    #[inline]
+    pub fn data_file(&self) -> &DataFile {
+        &self.data_file
+    }
 }
 
 /// Used to track additions and deletions in ManifestEntry.
@@ -924,34 +936,34 @@ impl TryFrom<i32> for ManifestStatus {
 }
 
 /// Data file carries data file path, partition tuple, metrics, …
-#[derive(Debug, PartialEq, Clone, Eq, TypedBuilder)]
+#[derive(Debug, PartialEq, Clone, Eq, Builder)]
 pub struct DataFile {
     /// field id: 134
     ///
     /// Type of content stored by the data file: data, equality deletes,
     /// or position deletes (all v1 files are data files)
-    content: DataContentType,
+    pub(crate) content: DataContentType,
     /// field id: 100
     ///
     /// Full URI for the file with FS scheme
-    file_path: String,
+    pub(crate) file_path: String,
     /// field id: 101
     ///
     /// String file format name, avro, orc or parquet
-    file_format: DataFileFormat,
+    pub(crate) file_format: DataFileFormat,
     /// field id: 102
     ///
     /// Partition data tuple, schema based on the partition spec output using
     /// partition field ids for the struct field ids
-    partition: Struct,
+    pub(crate) partition: Struct,
     /// field id: 103
     ///
     /// Number of records in this file
-    record_count: u64,
+    pub(crate) record_count: u64,
     /// field id: 104
     ///
     /// Total file size in bytes
-    file_size_in_bytes: u64,
+    pub(crate) file_size_in_bytes: u64,
     /// field id: 108
     /// key field id: 117
     /// value field id: 118
@@ -960,7 +972,7 @@ pub struct DataFile {
     /// store the column. Does not include bytes necessary to read other
     /// columns, like footers. Leave null for row-oriented formats (Avro)
     #[builder(default)]
-    column_sizes: HashMap<i32, u64>,
+    pub(crate) column_sizes: HashMap<i32, u64>,
     /// field id: 109
     /// key field id: 119
     /// value field id: 120
@@ -968,21 +980,21 @@ pub struct DataFile {
     /// Map from column id to number of values in the column (including null
     /// and NaN values)
     #[builder(default)]
-    value_counts: HashMap<i32, u64>,
+    pub(crate) value_counts: HashMap<i32, u64>,
     /// field id: 110
     /// key field id: 121
     /// value field id: 122
     ///
     /// Map from column id to number of null values in the column
     #[builder(default)]
-    null_value_counts: HashMap<i32, u64>,
+    pub(crate) null_value_counts: HashMap<i32, u64>,
     /// field id: 137
     /// key field id: 138
     /// value field id: 139
     ///
     /// Map from column id to number of NaN values in the column
     #[builder(default)]
-    nan_value_counts: HashMap<i32, u64>,
+    pub(crate) nan_value_counts: HashMap<i32, u64>,
     /// field id: 125
     /// key field id: 126
     /// value field id: 127
@@ -995,7 +1007,7 @@ pub struct DataFile {
     ///
     /// - [Binary single-value serialization](https://iceberg.apache.org/spec/#binary-single-value-serialization)
     #[builder(default)]
-    lower_bounds: HashMap<i32, Literal>,
+    pub(crate) lower_bounds: HashMap<i32, Datum>,
     /// field id: 128
     /// key field id: 129
     /// value field id: 130
@@ -1008,19 +1020,19 @@ pub struct DataFile {
     ///
     /// - [Binary single-value serialization](https://iceberg.apache.org/spec/#binary-single-value-serialization)
     #[builder(default)]
-    upper_bounds: HashMap<i32, Literal>,
+    pub(crate) upper_bounds: HashMap<i32, Datum>,
     /// field id: 131
     ///
     /// Implementation-specific key metadata for encryption
     #[builder(default)]
-    key_metadata: Vec<u8>,
+    pub(crate) key_metadata: Vec<u8>,
     /// field id: 132
     /// element field id: 133
     ///
     /// Split offsets for the data file. For example, all row group offsets
     /// in a Parquet file. Must be sorted ascending
     #[builder(default)]
-    split_offsets: Vec<i64>,
+    pub(crate) split_offsets: Vec<i64>,
     /// field id: 135
     /// element field id: 136
     ///
@@ -1029,7 +1041,7 @@ pub struct DataFile {
     /// otherwise. Fields with ids listed in this column must be present
     /// in the delete file
     #[builder(default)]
-    equality_ids: Vec<i32>,
+    pub(crate) equality_ids: Vec<i32>,
     /// field id: 140
     ///
     /// ID representing sort order for this file.
@@ -1041,9 +1053,92 @@ pub struct DataFile {
     /// order id to null. Readers must ignore sort order id for position
     /// delete files.
     #[builder(default, setter(strip_option))]
-    sort_order_id: Option<i32>,
+    pub(crate) sort_order_id: Option<i32>,
 }
 
+impl DataFile {
+    /// Get the content type of the data file (data, equality deletes, or position deletes)
+    pub fn content_type(&self) -> DataContentType {
+        self.content
+    }
+    /// Get the file path as full URI with FS scheme
+    pub fn file_path(&self) -> &str {
+        &self.file_path
+    }
+    /// Get the file format of the file (avro, orc or parquet).
+    pub fn file_format(&self) -> DataFileFormat {
+        self.file_format
+    }
+    /// Get the partition values of the file.
+    pub fn partition(&self) -> &Struct {
+        &self.partition
+    }
+    /// Get the record count in the data file.
+    pub fn record_count(&self) -> u64 {
+        self.record_count
+    }
+    /// Get the file size in bytes.
+    pub fn file_size_in_bytes(&self) -> u64 {
+        self.file_size_in_bytes
+    }
+    /// Get the column sizes.
+    /// Map from column id to the total size on disk of all regions that
+    /// store the column. Does not include bytes necessary to read other
+    /// columns, like footers. Null for row-oriented formats (Avro)
+    pub fn column_sizes(&self) -> &HashMap<i32, u64> {
+        &self.column_sizes
+    }
+    /// Get the columns value counts for the data file.
+    /// Map from column id to number of values in the column (including null
+    /// and NaN values)
+    pub fn value_counts(&self) -> &HashMap<i32, u64> {
+        &self.value_counts
+    }
+    /// Get the null value counts of the data file.
+    /// Map from column id to number of null values in the column
+    pub fn null_value_counts(&self) -> &HashMap<i32, u64> {
+        &self.null_value_counts
+    }
+    /// Get the nan value counts of the data file.
+    /// Map from column id to number of NaN values in the column
+    pub fn nan_value_counts(&self) -> &HashMap<i32, u64> {
+        &self.nan_value_counts
+    }
+    /// Get the lower bounds of the data file values per column.
+    /// Map from column id to lower bound in the column serialized as binary.
+    pub fn lower_bounds(&self) -> &HashMap<i32, Datum> {
+        &self.lower_bounds
+    }
+    /// Get the upper bounds of the data file values per column.
+    /// Map from column id to upper bound in the column serialized as binary.
+    pub fn upper_bounds(&self) -> &HashMap<i32, Datum> {
+        &self.upper_bounds
+    }
+    /// Get the Implementation-specific key metadata for the data file.
+    pub fn key_metadata(&self) -> &[u8] {
+        &self.key_metadata
+    }
+    /// Get the split offsets of the data file.
+    /// For example, all row group offsets in a Parquet file.
+    pub fn split_offsets(&self) -> &[i64] {
+        &self.split_offsets
+    }
+    /// Get the equality ids of the data file.
+    /// Field ids used to determine row equality in equality delete files.
+    /// null when content is not EqualityDeletes.
+    pub fn equality_ids(&self) -> &[i32] {
+        &self.equality_ids
+    }
+    /// Get the sort order id of the data file.
+    /// Only data files and equality delete files should be
+    /// written with a non-null order id. Position deletes are required to be
+    /// sorted by file and position, not a table order, and should set sort
+    /// order id to null. Readers must ignore sort order id for position
+    /// delete files.
+    pub fn sort_order_id(&self) -> Option<i32> {
+        self.sort_order_id
+    }
+}
 /// Type of content stored by the data file: data, equality deletes, or
 /// position deletes (all v1 files are data files)
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
@@ -1099,34 +1194,25 @@ impl FromStr for DataFileFormat {
     }
 }
 
-impl ToString for DataFileFormat {
-    fn to_string(&self) -> String {
+impl std::fmt::Display for DataFileFormat {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            DataFileFormat::Avro => "avro",
-            DataFileFormat::Orc => "orc",
-            DataFileFormat::Parquet => "parquet",
+            DataFileFormat::Avro => write!(f, "avro"),
+            DataFileFormat::Orc => write!(f, "orc"),
+            DataFileFormat::Parquet => write!(f, "parquet"),
         }
-        .to_string()
     }
 }
 
 mod _serde {
     use std::collections::HashMap;
 
-    use serde_bytes::ByteBuf;
     use serde_derive::{Deserialize, Serialize};
     use serde_with::serde_as;
 
-    use crate::spec::Literal;
-    use crate::spec::RawLiteral;
-    use crate::spec::Schema;
-    use crate::spec::Struct;
-    use crate::spec::StructType;
-    use crate::spec::Type;
-    use crate::Error;
-    use crate::ErrorKind;
-
     use super::ManifestEntry;
+    use crate::spec::{Datum, Literal, RawLiteral, Schema, Struct, StructType, Type};
+    use crate::{Error, ErrorKind};
 
     #[derive(Serialize, Deserialize)]
     pub(super) struct ManifestEntryV2 {
@@ -1325,23 +1411,32 @@ mod _serde {
     fn parse_bytes_entry(
         v: Vec<BytesEntry>,
         schema: &Schema,
-    ) -> Result<HashMap<i32, Literal>, Error> {
+    ) -> Result<HashMap<i32, Datum>, Error> {
         let mut m = HashMap::with_capacity(v.len());
         for entry in v {
             // We ignore the entry if the field is not found in the schema, due to schema evolution.
             if let Some(field) = schema.field_by_id(entry.key) {
-                let data_type = &field.field_type;
-                m.insert(entry.key, Literal::try_from_bytes(&entry.value, data_type)?);
+                let data_type = field
+                    .field_type
+                    .as_primitive_type()
+                    .ok_or_else(|| {
+                        Error::new(
+                            ErrorKind::DataInvalid,
+                            format!("field {} is not a primitive type", field.name),
+                        )
+                    })?
+                    .clone();
+                m.insert(entry.key, Datum::try_from_bytes(&entry.value, data_type)?);
             }
         }
         Ok(m)
     }
 
-    fn to_bytes_entry(v: HashMap<i32, Literal>) -> Vec<BytesEntry> {
+    fn to_bytes_entry(v: impl IntoIterator<Item = (i32, Datum)>) -> Vec<BytesEntry> {
         v.into_iter()
             .map(|e| BytesEntry {
                 key: e.0,
-                value: Into::<ByteBuf>::into(e.1),
+                value: e.1.to_bytes(),
             })
             .collect()
     }
@@ -1379,15 +1474,16 @@ mod _serde {
 
     #[cfg(test)]
     mod tests {
-        use crate::spec::manifest::_serde::{parse_i64_entry, I64Entry};
         use std::collections::HashMap;
 
+        use crate::spec::manifest::_serde::{parse_i64_entry, I64Entry};
+
         #[test]
         fn test_parse_negative_manifest_entry() {
-            let entries = vec![
-                I64Entry { key: 1, value: -1 },
-                I64Entry { key: 2, value: 3 },
-            ];
+            let entries = vec![I64Entry { key: 1, value: -1 }, I64Entry {
+                key: 2,
+                value: 3,
+            }];
 
             let ret = parse_i64_entry(entries).unwrap();
 
@@ -1400,17 +1496,13 @@ mod _serde {
 #[cfg(test)]
 mod tests {
     use std::fs;
+    use std::sync::Arc;
 
     use tempfile::TempDir;
 
     use super::*;
     use crate::io::FileIOBuilder;
-    use crate::spec::NestedField;
-    use crate::spec::PrimitiveType;
-    use crate::spec::Struct;
-    use crate::spec::Transform;
-    use crate::spec::Type;
-    use std::sync::Arc;
+    use crate::spec::{Literal, NestedField, PrimitiveType, Struct, Transform, Type};
 
     #[tokio::test]
     async fn test_parse_manifest_v2_unpartition() {
@@ -1734,8 +1826,8 @@ mod tests {
                     value_counts: HashMap::from([(1,1),(2,1),(3,1)]),
                     null_value_counts: HashMap::from([(1,0),(2,0),(3,0)]),
                     nan_value_counts: HashMap::new(),
-                    lower_bounds: HashMap::from([(1,Literal::int(1)),(2,Literal::string("a")),(3,Literal::string("AC/DC"))]),
-                    upper_bounds: HashMap::from([(1,Literal::int(1)),(2,Literal::string("a")),(3,Literal::string("AC/DC"))]),
+                    lower_bounds: HashMap::from([(1,Datum::int(1)),(2,Datum::string("a")),(3,Datum::string("AC/DC"))]),
+                    upper_bounds: HashMap::from([(1,Datum::int(1)),(2,Datum::string("a")),(3,Datum::string("AC/DC"))]),
                     key_metadata: vec![],
                     split_offsets: vec![4],
                     equality_ids: vec![],
@@ -1800,8 +1892,7 @@ mod tests {
                         partition: Struct::from_iter(
                             vec![
                                 Some(
-                                    Literal::try_from_bytes(&[120], &Type::Primitive(PrimitiveType::String))
-                                        .unwrap()
+                                    Literal::string("x"),
                                 ),
                             ]
                                 .into_iter()
@@ -1813,14 +1904,14 @@ mod tests {
                         null_value_counts: HashMap::from([(1, 0), (2, 0), (3, 0)]),
                         nan_value_counts: HashMap::new(),
                         lower_bounds: HashMap::from([
-                        (1, Literal::long(1)),
-                        (2, Literal::string("a")),
-                        (3, Literal::string("x"))
+                        (1, Datum::long(1)),
+                        (2, Datum::string("a")),
+                        (3, Datum::string("x"))
                         ]),
                         upper_bounds: HashMap::from([
-                        (1, Literal::long(1)),
-                        (2, Literal::string("a")),
-                        (3, Literal::string("x"))
+                        (1, Datum::long(1)),
+                        (2, Datum::string("a")),
+                        (3, Datum::string("x"))
                         ]),
                         key_metadata: vec![],
                         split_offsets: vec![4],
@@ -1836,8 +1927,8 @@ mod tests {
         let entry = test_manifest_read_write(manifest, writer).await;
 
         assert_eq!(entry.partitions.len(), 1);
-        assert_eq!(entry.partitions[0].lower_bound, Some(Literal::string("x")));
-        assert_eq!(entry.partitions[0].upper_bound, Some(Literal::string("x")));
+        assert_eq!(entry.partitions[0].lower_bound, Some(Datum::string("x")));
+        assert_eq!(entry.partitions[0].upper_bound, Some(Datum::string("x")));
     }
 
     #[tokio::test]
@@ -1888,14 +1979,14 @@ mod tests {
                     null_value_counts: HashMap::default(),
                     nan_value_counts: HashMap::new(),
                     lower_bounds: HashMap::from([
-                        (1, Literal::long(1)),
-                        (2, Literal::int(2)),
-                        (3, Literal::string("x"))
+                        (1, Datum::long(1)),
+                        (2, Datum::int(2)),
+                        (3, Datum::string("x"))
                     ]),
                     upper_bounds: HashMap::from([
-                        (1, Literal::long(1)),
-                        (2, Literal::int(2)),
-                        (3, Literal::string("x"))
+                        (1, Datum::long(1)),
+                        (2, Datum::int(2)),
+                        (3, Datum::string("x"))
                     ]),
                     key_metadata: vec![],
                     split_offsets: vec![4],
@@ -1960,12 +2051,12 @@ mod tests {
                     null_value_counts: HashMap::default(),
                     nan_value_counts: HashMap::new(),
                     lower_bounds: HashMap::from([
-                        (1, Literal::long(1)),
-                        (2, Literal::int(2)),
+                        (1, Datum::long(1)),
+                        (2, Datum::int(2)),
                     ]),
                     upper_bounds: HashMap::from([
-                        (1, Literal::long(1)),
-                        (2, Literal::int(2)),
+                        (1, Datum::long(1)),
+                        (2, Datum::int(2)),
                     ]),
                     key_metadata: vec![],
                     split_offsets: vec![4],
@@ -1981,7 +2072,7 @@ mod tests {
     async fn test_manifest_read_write(
         manifest: Manifest,
         writer_builder: impl FnOnce(OutputFile) -> ManifestWriter,
-    ) -> ManifestListEntry {
+    ) -> ManifestFile {
         let (bs, res) = write_manifest(&manifest, writer_builder).await;
         let actual_manifest = Manifest::parse_avro(bs.as_slice()).unwrap();
 
@@ -1993,7 +2084,7 @@ mod tests {
     async fn write_manifest(
         manifest: &Manifest,
         writer_builder: impl FnOnce(OutputFile) -> ManifestWriter,
-    ) -> (Vec<u8>, ManifestListEntry) {
+    ) -> (Vec<u8>, ManifestFile) {
         let temp_dir = TempDir::new().unwrap();
         let path = temp_dir.path().join("test_manifest.avro");
         let io = FileIOBuilder::new_fs_io().build().unwrap();
