@@ -18,24 +18,26 @@
 //! Defines the [table metadata](https://iceberg.apache.org/spec/#table-metadata).
 //! The main struct here is [TableMetadataV2] which defines the data for a table.
 
-use serde::{Deserialize, Serialize};
-use serde_repr::{Deserialize_repr, Serialize_repr};
 use std::cmp::Ordering;
+use std::collections::HashMap;
 use std::fmt::{Display, Formatter};
-use std::{collections::HashMap, sync::Arc};
-use uuid::Uuid;
-
-use super::{
-    snapshot::{Snapshot, SnapshotReference, SnapshotRetention},
-    PartitionSpecRef, SchemaId, SchemaRef, SnapshotRef, SortOrderRef,
-};
+use std::sync::Arc;
 
 use _serde::TableMetadataEnum;
+use chrono::{DateTime, Utc};
+use serde::{Deserialize, Serialize};
+use serde_repr::{Deserialize_repr, Serialize_repr};
+use uuid::Uuid;
 
-use chrono::{DateTime, TimeZone, Utc};
+use super::snapshot::{Snapshot, SnapshotReference, SnapshotRetention};
+use super::{
+    PartitionSpec, PartitionSpecRef, SchemaId, SchemaRef, SnapshotRef, SortOrder, SortOrderRef,
+    DEFAULT_PARTITION_SPEC_ID,
+};
+use crate::error::{timestamp_ms_to_utc, Result};
+use crate::{Error, ErrorKind, TableCreation};
 
 static MAIN_BRANCH: &str = "main";
-static DEFAULT_SPEC_ID: i32 = 0;
 static DEFAULT_SORT_ORDER_ID: i64 = 0;
 
 pub(crate) static EMPTY_SNAPSHOT_ID: i64 = -1;
@@ -52,46 +54,46 @@ pub type TableMetadataRef = Arc<TableMetadata>;
 /// We check the validity of this data structure when constructing.
 pub struct TableMetadata {
     /// Integer Version for the format.
-    format_version: FormatVersion,
+    pub(crate) format_version: FormatVersion,
     /// A UUID that identifies the table
-    table_uuid: Uuid,
+    pub(crate) table_uuid: Uuid,
     /// Location tables base location
-    location: String,
+    pub(crate) location: String,
     /// The tables highest sequence number
-    last_sequence_number: i64,
+    pub(crate) last_sequence_number: i64,
     /// Timestamp in milliseconds from the unix epoch when the table was last updated.
-    last_updated_ms: i64,
+    pub(crate) last_updated_ms: i64,
     /// An integer; the highest assigned column ID for the table.
-    last_column_id: i32,
+    pub(crate) last_column_id: i32,
     /// A list of schemas, stored as objects with schema-id.
-    schemas: HashMap<i32, SchemaRef>,
+    pub(crate) schemas: HashMap<i32, SchemaRef>,
     /// ID of the table’s current schema.
-    current_schema_id: i32,
+    pub(crate) current_schema_id: i32,
     /// A list of partition specs, stored as full partition spec objects.
-    partition_specs: HashMap<i32, PartitionSpecRef>,
+    pub(crate) partition_specs: HashMap<i32, PartitionSpecRef>,
     /// ID of the “current” spec that writers should use by default.
-    default_spec_id: i32,
+    pub(crate) default_spec_id: i32,
     /// An integer; the highest assigned partition field ID across all partition specs for the table.
-    last_partition_id: i32,
+    pub(crate) last_partition_id: i32,
     ///A string to string map of table properties. This is used to control settings that
     /// affect reading and writing and is not intended to be used for arbitrary metadata.
     /// For example, commit.retry.num-retries is used to control the number of commit retries.
-    properties: HashMap<String, String>,
+    pub(crate) properties: HashMap<String, String>,
     /// long ID of the current table snapshot; must be the same as the current
     /// ID of the main branch in refs.
-    current_snapshot_id: Option<i64>,
+    pub(crate) current_snapshot_id: Option<i64>,
     ///A list of valid snapshots. Valid snapshots are snapshots for which all
     /// data files exist in the file system. A data file must not be deleted
     /// from the file system until the last snapshot in which it was listed is
     /// garbage collected.
-    snapshots: HashMap<i64, SnapshotRef>,
+    pub(crate) snapshots: HashMap<i64, SnapshotRef>,
     /// A list (optional) of timestamp and snapshot ID pairs that encodes changes
     /// to the current snapshot for the table. Each time the current-snapshot-id
     /// is changed, a new entry should be added with the last-updated-ms
     /// and the new current-snapshot-id. When snapshots are expired from
     /// the list of valid snapshots, all entries before a snapshot that has
     /// expired should be removed.
-    snapshot_log: Vec<SnapshotLog>,
+    pub(crate) snapshot_log: Vec<SnapshotLog>,
 
     /// A list (optional) of timestamp and metadata file location pairs
     /// that encodes changes to the previous metadata files for the table.
@@ -99,19 +101,19 @@ pub struct TableMetadata {
     /// previous metadata file location should be added to the list.
     /// Tables can be configured to remove oldest metadata log entries and
     /// keep a fixed-size log of the most recent entries after a commit.
-    metadata_log: Vec<MetadataLog>,
+    pub(crate) metadata_log: Vec<MetadataLog>,
 
     /// A list of sort orders, stored as full sort order objects.
-    sort_orders: HashMap<i64, SortOrderRef>,
+    pub(crate) sort_orders: HashMap<i64, SortOrderRef>,
     /// Default sort order id of the table. Note that this could be used by
     /// writers, but is not used when reading because reads use the specs
     /// stored in manifest files.
-    default_sort_order_id: i64,
+    pub(crate) default_sort_order_id: i64,
     ///A map of snapshot references. The map keys are the unique snapshot reference
     /// names in the table, and the map values are snapshot reference objects.
     /// There is always a main branch reference pointing to the current-snapshot-id
     /// even if the refs map is null.
-    refs: HashMap<String, SnapshotReference>,
+    pub(crate) refs: HashMap<String, SnapshotReference>,
 }
 
 impl TableMetadata {
@@ -141,8 +143,14 @@ impl TableMetadata {
 
     /// Returns last updated time.
     #[inline]
-    pub fn last_updated_ms(&self) -> DateTime<Utc> {
-        Utc.timestamp_millis_opt(self.last_updated_ms).unwrap()
+    pub fn last_updated_timestamp(&self) -> Result<DateTime<Utc>> {
+        timestamp_ms_to_utc(self.last_updated_ms)
+    }
+
+    /// Returns last updated time in milliseconds.
+    #[inline]
+    pub fn last_updated_ms(&self) -> i64 {
+        self.last_updated_ms
     }
 
     /// Returns schemas
@@ -179,8 +187,8 @@ impl TableMetadata {
     /// Get default partition spec
     #[inline]
     pub fn default_partition_spec(&self) -> Option<&PartitionSpecRef> {
-        if self.default_spec_id == DEFAULT_SPEC_ID {
-            self.partition_spec_by_id(DEFAULT_SPEC_ID)
+        if self.default_spec_id == DEFAULT_PARTITION_SPEC_ID {
+            self.partition_spec_by_id(DEFAULT_PARTITION_SPEC_ID)
         } else {
             Some(
                 self.partition_spec_by_id(self.default_spec_id)
@@ -259,14 +267,11 @@ impl TableMetadata {
                 s.snapshot_id = snapshot.snapshot_id();
             })
             .or_insert_with(|| {
-                SnapshotReference::new(
-                    snapshot.snapshot_id(),
-                    SnapshotRetention::Branch {
-                        min_snapshots_to_keep: None,
-                        max_snapshot_age_ms: None,
-                        max_ref_age_ms: None,
-                    },
-                )
+                SnapshotReference::new(snapshot.snapshot_id(), SnapshotRetention::Branch {
+                    min_snapshots_to_keep: None,
+                    max_snapshot_age_ms: None,
+                    max_ref_age_ms: None,
+                })
             });
 
         self.snapshot_log.push(snapshot.log());
@@ -275,31 +280,127 @@ impl TableMetadata {
     }
 }
 
+/// Manipulating table metadata.
+pub struct TableMetadataBuilder(TableMetadata);
+
+impl TableMetadataBuilder {
+    /// Creates a new table metadata builder from the given table metadata.
+    pub fn new(origin: TableMetadata) -> Self {
+        Self(origin)
+    }
+
+    /// Creates a new table metadata builder from the given table creation.
+    pub fn from_table_creation(table_creation: TableCreation) -> Result<Self> {
+        let TableCreation {
+            name: _,
+            location,
+            schema,
+            partition_spec,
+            sort_order,
+            properties,
+        } = table_creation;
+
+        let partition_specs = match partition_spec {
+            Some(_) => {
+                return Err(Error::new(
+                    ErrorKind::FeatureUnsupported,
+                    "Can't create table with partition spec now",
+                ))
+            }
+            None => HashMap::from([(
+                DEFAULT_PARTITION_SPEC_ID,
+                Arc::new(PartitionSpec {
+                    spec_id: DEFAULT_PARTITION_SPEC_ID,
+                    fields: vec![],
+                }),
+            )]),
+        };
+
+        let sort_orders = match sort_order {
+            Some(_) => {
+                return Err(Error::new(
+                    ErrorKind::FeatureUnsupported,
+                    "Can't create table with sort order now",
+                ))
+            }
+            None => HashMap::from([(
+                DEFAULT_SORT_ORDER_ID,
+                Arc::new(SortOrder {
+                    order_id: DEFAULT_SORT_ORDER_ID,
+                    fields: vec![],
+                }),
+            )]),
+        };
+
+        let table_metadata = TableMetadata {
+            format_version: FormatVersion::V2,
+            table_uuid: Uuid::now_v7(),
+            location: location.ok_or_else(|| {
+                Error::new(
+                    ErrorKind::DataInvalid,
+                    "Can't create table without location",
+                )
+            })?,
+            last_sequence_number: 0,
+            last_updated_ms: Utc::now().timestamp_millis(),
+            last_column_id: schema.highest_field_id(),
+            current_schema_id: schema.schema_id(),
+            schemas: HashMap::from([(schema.schema_id(), Arc::new(schema))]),
+            partition_specs,
+            default_spec_id: DEFAULT_PARTITION_SPEC_ID,
+            last_partition_id: 0,
+            properties,
+            current_snapshot_id: None,
+            snapshots: Default::default(),
+            snapshot_log: vec![],
+            sort_orders,
+            metadata_log: vec![],
+            default_sort_order_id: DEFAULT_SORT_ORDER_ID,
+            refs: Default::default(),
+        };
+
+        Ok(Self(table_metadata))
+    }
+
+    /// Changes uuid of table metadata.
+    pub fn assign_uuid(mut self, uuid: Uuid) -> Result<Self> {
+        self.0.table_uuid = uuid;
+        Ok(self)
+    }
+
+    /// Returns the new table metadata after changes.
+    pub fn build(self) -> Result<TableMetadata> {
+        Ok(self.0)
+    }
+}
+
 pub(super) mod _serde {
     /// This is a helper module that defines types to help with serialization/deserialization.
     /// For deserialization the input first gets read into either the [TableMetadataV1] or [TableMetadataV2] struct
     /// and then converted into the [TableMetadata] struct. Serialization works the other way around.
     /// [TableMetadataV1] and [TableMetadataV2] are internal struct that are only used for serialization and deserialization.
-    use std::{collections::HashMap, sync::Arc};
+    use std::collections::HashMap;
+    /// This is a helper module that defines types to help with serialization/deserialization.
+    /// For deserialization the input first gets read into either the [TableMetadataV1] or [TableMetadataV2] struct
+    /// and then converted into the [TableMetadata] struct. Serialization works the other way around.
+    /// [TableMetadataV1] and [TableMetadataV2] are internal struct that are only used for serialization and deserialization.
+    use std::sync::Arc;
 
     use itertools::Itertools;
     use serde::{Deserialize, Serialize};
     use uuid::Uuid;
 
-    use crate::spec::{Snapshot, EMPTY_SNAPSHOT_ID};
-    use crate::{
-        spec::{
-            schema::_serde::{SchemaV1, SchemaV2},
-            snapshot::_serde::{SnapshotV1, SnapshotV2},
-            PartitionField, PartitionSpec, Schema, SnapshotReference, SnapshotRetention, SortOrder,
-        },
-        Error, ErrorKind,
-    };
-
     use super::{
-        FormatVersion, MetadataLog, SnapshotLog, TableMetadata, DEFAULT_SORT_ORDER_ID,
-        DEFAULT_SPEC_ID, MAIN_BRANCH,
+        FormatVersion, MetadataLog, SnapshotLog, TableMetadata, DEFAULT_PARTITION_SPEC_ID,
+        DEFAULT_SORT_ORDER_ID, MAIN_BRANCH,
     };
+    use crate::spec::schema::_serde::{SchemaV1, SchemaV2};
+    use crate::spec::snapshot::_serde::{SnapshotV1, SnapshotV2};
+    use crate::spec::{
+        PartitionField, PartitionSpec, Schema, Snapshot, SnapshotReference, SnapshotRetention,
+        SortOrder, EMPTY_SNAPSHOT_ID,
+    };
+    use crate::{Error, ErrorKind};
 
     #[derive(Debug, Serialize, Deserialize, PartialEq, Eq)]
     #[serde(untagged)]
@@ -377,13 +478,11 @@ pub(super) mod _serde {
 
     /// Helper to serialize and deserialize the format version.
     #[derive(Debug, PartialEq, Eq)]
-    pub(super) struct VersionNumber<const V: u8>;
+    pub(crate) struct VersionNumber<const V: u8>;
 
     impl Serialize for TableMetadata {
         fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-        where
-            S: serde::Serializer,
-        {
+        where S: serde::Serializer {
             // we must do a clone here
             let table_metadata_enum: TableMetadataEnum =
                 self.clone().try_into().map_err(serde::ser::Error::custom)?;
@@ -394,18 +493,14 @@ pub(super) mod _serde {
 
     impl<const V: u8> Serialize for VersionNumber<V> {
         fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-        where
-            S: serde::Serializer,
-        {
+        where S: serde::Serializer {
             serializer.serialize_u8(V)
         }
     }
 
     impl<'de, const V: u8> Deserialize<'de> for VersionNumber<V> {
         fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-        where
-            D: serde::Deserializer<'de>,
-        {
+        where D: serde::Deserializer<'de> {
             let value = u8::deserialize(deserializer)?;
             if value == V {
                 Ok(VersionNumber::<V>)
@@ -473,7 +568,7 @@ pub(super) mod _serde {
                     value
                         .partition_specs
                         .into_iter()
-                        .map(|x| (x.spec_id, Arc::new(x))),
+                        .map(|x| (x.spec_id(), Arc::new(x))),
                 ),
                 default_spec_id: value.default_spec_id,
                 last_partition_id: value.last_partition_id,
@@ -500,17 +595,14 @@ pub(super) mod _serde {
                 default_sort_order_id: value.default_sort_order_id,
                 refs: value.refs.unwrap_or_else(|| {
                     if let Some(snapshot_id) = current_snapshot_id {
-                        HashMap::from_iter(vec![(
-                            MAIN_BRANCH.to_string(),
-                            SnapshotReference {
-                                snapshot_id,
-                                retention: SnapshotRetention::Branch {
-                                    min_snapshots_to_keep: None,
-                                    max_snapshot_age_ms: None,
-                                    max_ref_age_ms: None,
-                                },
+                        HashMap::from_iter(vec![(MAIN_BRANCH.to_string(), SnapshotReference {
+                            snapshot_id,
+                            retention: SnapshotRetention::Branch {
+                                min_snapshots_to_keep: None,
+                                max_snapshot_age_ms: None,
+                                max_ref_age_ms: None,
                             },
-                        )])
+                        })])
                     } else {
                         HashMap::new()
                     }
@@ -551,12 +643,12 @@ pub(super) mod _serde {
                     .partition_specs
                     .unwrap_or_else(|| {
                         vec![PartitionSpec {
-                            spec_id: DEFAULT_SPEC_ID,
+                            spec_id: DEFAULT_PARTITION_SPEC_ID,
                             fields: value.partition_spec,
                         }]
                     })
                     .into_iter()
-                    .map(|x| (x.spec_id, Arc::new(x))),
+                    .map(|x| (x.spec_id(), Arc::new(x))),
             );
             Ok(TableMetadata {
                 format_version: FormatVersion::V1,
@@ -608,17 +700,14 @@ pub(super) mod _serde {
                     None => HashMap::new(),
                 },
                 default_sort_order_id: value.default_sort_order_id.unwrap_or(DEFAULT_SORT_ORDER_ID),
-                refs: HashMap::from_iter(vec![(
-                    MAIN_BRANCH.to_string(),
-                    SnapshotReference {
-                        snapshot_id: value.current_snapshot_id.unwrap_or_default(),
-                        retention: SnapshotRetention::Branch {
-                            min_snapshots_to_keep: None,
-                            max_snapshot_age_ms: None,
-                            max_ref_age_ms: None,
-                        },
+                refs: HashMap::from_iter(vec![(MAIN_BRANCH.to_string(), SnapshotReference {
+                    snapshot_id: value.current_snapshot_id.unwrap_or_default(),
+                    retention: SnapshotRetention::Branch {
+                        min_snapshots_to_keep: None,
+                        max_snapshot_age_ms: None,
+                        max_ref_age_ms: None,
                     },
-                )]),
+                })]),
             })
         }
     }
@@ -649,14 +738,10 @@ pub(super) mod _serde {
                     .collect(),
                 default_spec_id: v.default_spec_id,
                 last_partition_id: v.last_partition_id,
-                properties: if v.properties.is_empty() {
-                    None
-                } else {
-                    Some(v.properties)
-                },
+                properties: Some(v.properties),
                 current_snapshot_id: v.current_snapshot_id.or(Some(-1)),
                 snapshots: if v.snapshots.is_empty() {
-                    None
+                    Some(vec![])
                 } else {
                     Some(
                         v.snapshots
@@ -723,7 +808,7 @@ pub(super) mod _serde {
                 partition_spec: v
                     .partition_specs
                     .get(&v.default_spec_id)
-                    .map(|x| x.fields.clone())
+                    .map(|x| x.fields().to_vec())
                     .unwrap_or_default(),
                 partition_specs: Some(
                     v.partition_specs
@@ -824,27 +909,35 @@ pub struct SnapshotLog {
 
 impl SnapshotLog {
     /// Returns the last updated timestamp as a DateTime<Utc> with millisecond precision
-    pub fn timestamp(self) -> DateTime<Utc> {
-        Utc.timestamp_millis_opt(self.timestamp_ms).unwrap()
+    pub fn timestamp(self) -> Result<DateTime<Utc>> {
+        timestamp_ms_to_utc(self.timestamp_ms)
+    }
+
+    /// Returns the timestamp in milliseconds
+    #[inline]
+    pub fn timestamp_ms(&self) -> i64 {
+        self.timestamp_ms
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use std::{collections::HashMap, fs, sync::Arc};
+    use std::collections::HashMap;
+    use std::fs;
+    use std::sync::Arc;
 
     use anyhow::Result;
+    use pretty_assertions::assert_eq;
     use uuid::Uuid;
 
-    use pretty_assertions::assert_eq;
-
+    use super::{FormatVersion, MetadataLog, SnapshotLog, TableMetadataBuilder};
+    use crate::spec::table_metadata::TableMetadata;
     use crate::spec::{
-        table_metadata::TableMetadata, ManifestListLocation, NestedField, NullOrder, Operation,
-        PartitionField, PartitionSpec, PrimitiveType, Schema, Snapshot, SnapshotReference,
-        SnapshotRetention, SortDirection, SortField, SortOrder, Summary, Transform, Type,
+        NestedField, NullOrder, Operation, PartitionField, PartitionSpec, PrimitiveType, Schema,
+        Snapshot, SnapshotReference, SnapshotRetention, SortDirection, SortField, SortOrder,
+        Summary, Transform, Type, UnboundPartitionField,
     };
-
-    use super::{FormatVersion, MetadataLog, SnapshotLog};
+    use crate::TableCreation;
 
     fn check_table_metadata_serde(json: &str, expected_type: TableMetadata) {
         let desered_type: TableMetadata = serde_json::from_str(json).unwrap();
@@ -927,16 +1020,15 @@ mod tests {
             .build()
             .unwrap();
 
-        let partition_spec = PartitionSpec::builder()
-            .with_spec_id(1)
-            .with_partition_field(PartitionField {
+        let partition_spec = PartitionSpec {
+            spec_id: 1,
+            fields: vec![PartitionField {
                 name: "ts_day".to_string(),
                 transform: Transform::Day,
                 source_id: 4,
                 field_id: 1000,
-            })
-            .build()
-            .unwrap();
+            }],
+        };
 
         let expected = TableMetadata {
             format_version: FormatVersion::V2,
@@ -1086,25 +1178,24 @@ mod tests {
             .build()
             .unwrap();
 
-        let partition_spec = PartitionSpec::builder()
+        let partition_spec = PartitionSpec::builder(&schema)
             .with_spec_id(0)
-            .with_partition_field(PartitionField {
-                name: "vendor_id".to_string(),
-                transform: Transform::Identity,
-                source_id: 1,
-                field_id: 1000,
-            })
+            .add_partition_field("vendor_id", "vendor_id", Transform::Identity)
+            .unwrap()
             .build()
             .unwrap();
 
-        let sort_order = SortOrder::builder().with_order_id(0).build().unwrap();
+        let sort_order = SortOrder::builder()
+            .with_order_id(0)
+            .build_unbound()
+            .unwrap();
 
         let snapshot = Snapshot::builder()
             .with_snapshot_id(638933773299822130)
             .with_timestamp_ms(1662532818843)
             .with_sequence_number(0)
             .with_schema_id(0)
-            .with_manifest_list(ManifestListLocation::ManifestListFile("/home/iceberg/warehouse/nyc/taxis/metadata/snap-638933773299822130-1-7e6760f0-4f6c-4b23-b907-0a5a174e3863.avro".to_string()))
+            .with_manifest_list("/home/iceberg/warehouse/nyc/taxis/metadata/snap-638933773299822130-1-7e6760f0-4f6c-4b23-b907-0a5a174e3863.avro")
             .with_summary(Summary { operation: Operation::Append, other: HashMap::from_iter(vec![("spark.app.id".to_string(), "local-1662532784305".to_string()), ("added-data-files".to_string(), "4".to_string()), ("added-records".to_string(), "4".to_string()), ("added-files-size".to_string(), "6001".to_string())]) })
             .build();
 
@@ -1196,14 +1287,15 @@ mod tests {
             .build()
             .unwrap();
 
-        let partition_spec = PartitionSpec::builder()
+        let partition_spec = PartitionSpec::builder(&schema1)
             .with_spec_id(0)
-            .with_partition_field(PartitionField {
+            .add_unbound_field(UnboundPartitionField {
                 name: "x".to_string(),
                 transform: Transform::Identity,
                 source_id: 1,
-                field_id: 1000,
+                partition_id: Some(1000),
             })
+            .unwrap()
             .build()
             .unwrap();
 
@@ -1221,16 +1313,14 @@ mod tests {
                 direction: SortDirection::Descending,
                 null_order: NullOrder::Last,
             })
-            .build()
+            .build_unbound()
             .unwrap();
 
         let snapshot1 = Snapshot::builder()
             .with_snapshot_id(3051729675574597004)
             .with_timestamp_ms(1515100955770)
             .with_sequence_number(0)
-            .with_manifest_list(ManifestListLocation::ManifestListFile(
-                "s3://a/b/1.avro".to_string(),
-            ))
+            .with_manifest_list("s3://a/b/1.avro")
             .with_summary(Summary {
                 operation: Operation::Append,
                 other: HashMap::new(),
@@ -1243,9 +1333,7 @@ mod tests {
             .with_timestamp_ms(1555100955770)
             .with_sequence_number(1)
             .with_schema_id(1)
-            .with_manifest_list(ManifestListLocation::ManifestListFile(
-                "s3://a/b/2.avro".to_string(),
-            ))
+            .with_manifest_list("s3://a/b/2.avro")
             .with_summary(Summary {
                 operation: Operation::Append,
                 other: HashMap::new(),
@@ -1283,17 +1371,14 @@ mod tests {
                 },
             ],
             metadata_log: Vec::new(),
-            refs: HashMap::from_iter(vec![(
-                "main".to_string(),
-                SnapshotReference {
-                    snapshot_id: 3055729675574597004,
-                    retention: SnapshotRetention::Branch {
-                        min_snapshots_to_keep: None,
-                        max_snapshot_age_ms: None,
-                        max_ref_age_ms: None,
-                    },
+            refs: HashMap::from_iter(vec![("main".to_string(), SnapshotReference {
+                snapshot_id: 3055729675574597004,
+                retention: SnapshotRetention::Branch {
+                    min_snapshots_to_keep: None,
+                    max_snapshot_age_ms: None,
+                    max_ref_age_ms: None,
                 },
-            )]),
+            })]),
         };
 
         check_table_metadata_serde(&metadata, expected);
@@ -1325,14 +1410,15 @@ mod tests {
             .build()
             .unwrap();
 
-        let partition_spec = PartitionSpec::builder()
+        let partition_spec = PartitionSpec::builder(&schema)
             .with_spec_id(0)
-            .with_partition_field(PartitionField {
+            .add_unbound_field(UnboundPartitionField {
                 name: "x".to_string(),
                 transform: Transform::Identity,
                 source_id: 1,
-                field_id: 1000,
+                partition_id: Some(1000),
             })
+            .unwrap()
             .build()
             .unwrap();
 
@@ -1350,7 +1436,7 @@ mod tests {
                 direction: SortDirection::Descending,
                 null_order: NullOrder::Last,
             })
-            .build()
+            .build_unbound()
             .unwrap();
 
         let expected = TableMetadata {
@@ -1404,14 +1490,15 @@ mod tests {
             .build()
             .unwrap();
 
-        let partition_spec = PartitionSpec::builder()
+        let partition_spec = PartitionSpec::builder(&schema)
             .with_spec_id(0)
-            .with_partition_field(PartitionField {
+            .add_unbound_field(UnboundPartitionField {
                 name: "x".to_string(),
                 transform: Transform::Identity,
                 source_id: 1,
-                field_id: 1000,
+                partition_id: Some(1000),
             })
+            .unwrap()
             .build()
             .unwrap();
 
@@ -1434,17 +1521,14 @@ mod tests {
             properties: HashMap::new(),
             snapshot_log: vec![],
             metadata_log: Vec::new(),
-            refs: HashMap::from_iter(vec![(
-                "main".to_string(),
-                SnapshotReference {
-                    snapshot_id: -1,
-                    retention: SnapshotRetention::Branch {
-                        min_snapshots_to_keep: None,
-                        max_snapshot_age_ms: None,
-                        max_ref_age_ms: None,
-                    },
+            refs: HashMap::from_iter(vec![("main".to_string(), SnapshotReference {
+                snapshot_id: -1,
+                retention: SnapshotRetention::Branch {
+                    min_snapshots_to_keep: None,
+                    max_snapshot_age_ms: None,
+                    max_ref_age_ms: None,
                 },
-            )]),
+            })]),
         };
 
         check_table_metadata_serde(&metadata, expected);
@@ -1570,4 +1654,66 @@ mod tests {
             table_meta_data.sort_orders.get(&default_sort_order_id)
         )
     }
+
+    #[test]
+    fn test_table_metadata_builder_from_table_creation() {
+        let table_creation = TableCreation::builder()
+            .location("s3://db/table".to_string())
+            .name("table".to_string())
+            .properties(HashMap::new())
+            .schema(Schema::builder().build().unwrap())
+            .build();
+        let table_metadata = TableMetadataBuilder::from_table_creation(table_creation)
+            .unwrap()
+            .build()
+            .unwrap();
+        assert_eq!(table_metadata.location, "s3://db/table");
+        assert_eq!(table_metadata.schemas.len(), 1);
+        assert_eq!(
+            table_metadata
+                .schemas
+                .get(&0)
+                .unwrap()
+                .as_struct()
+                .fields()
+                .len(),
+            0
+        );
+        assert_eq!(table_metadata.properties.len(), 0);
+        assert_eq!(
+            table_metadata.partition_specs,
+            HashMap::from([(
+                0,
+                Arc::new(
+                    PartitionSpec::builder(table_metadata.schemas.get(&0).unwrap())
+                        .with_spec_id(0)
+                        .build()
+                        .unwrap()
+                )
+            )])
+        );
+        assert_eq!(
+            table_metadata.sort_orders,
+            HashMap::from([(
+                0,
+                Arc::new(SortOrder {
+                    order_id: 0,
+                    fields: vec![]
+                })
+            )])
+        );
+    }
+
+    #[test]
+    fn test_table_builder_from_table_metadata() {
+        let table_metadata = get_test_table_metadata("TableMetadataV2Valid.json");
+        let table_metadata_builder = TableMetadataBuilder::new(table_metadata);
+        let uuid = Uuid::new_v4();
+        let table_metadata = table_metadata_builder
+            .assign_uuid(uuid)
+            .unwrap()
+            .build()
+            .unwrap();
+        assert_eq!(table_metadata.uuid(), uuid);
+    }
 }
